@@ -32,6 +32,9 @@
 #include <epan/prefs.h>
 #include <epan/conversation.h>
 #include <epan/tap.h>
+#include <epan/emem.h>
+#include <epan/strutil.h>
+#include <epan/asn1.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -40,6 +43,7 @@
 #include "packet-per.h"
 #include "packet-ranap.h"
 #include "packet-e212.h"
+#include "packet-sccp.h"
 
 #define SCCP_SSN_RANAP 0x8E
 
@@ -77,6 +81,7 @@ static guint type_of_message;
 static guint32 ProcedureCode;
 static guint32 ProtocolIE_ID;
 
+
 static int dissect_ranap_ies(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
 static int dissect_ranap_FirstValue_ies(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
 static int dissect_ranap_SecondValue_ies(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_tree *tree);
@@ -556,6 +561,7 @@ static int dissect_ranap_ies(tvbuff_t *tvb, int offset, asn1_ctx_t *actx, proto_
 			break;
 			
 	}
+	
 	return offset;
 }
 
@@ -1069,13 +1075,27 @@ dissect_ranap(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	/* make entry in the Protocol column on summary display */
 	if (check_col(pinfo->cinfo, COL_PROTOCOL))
 		col_set_str(pinfo->cinfo, COL_PROTOCOL, "RANAP");
+	
 
     /* create the ranap protocol tree */
     ranap_item = proto_tree_add_item(tree, proto_ranap, tvb, 0, -1, FALSE);
     ranap_tree = proto_item_add_subtree(ranap_item, ett_ranap);
 
+	ProcedureCode = 0xFFFFFFFF;
 	offset = dissect_RANAP_PDU_PDU(tvb, pinfo, ranap_tree);
 
+	if (pinfo->sccp_info) {
+		sccp_msg_info_t* sccp_msg = pinfo->sccp_info;
+		
+		if (sccp_msg->data.co.assoc)
+			sccp_msg->data.co.assoc->payload = SCCP_PLOAD_RANAP;
+		
+		if (! sccp_msg->data.co.label && ProcedureCode != 0xFFFFFFFF) {
+			const gchar* str = val_to_str(ProcedureCode, ranap_ProcedureCode_vals,"Unknown RANAP");
+			sccp_msg->data.co.label = se_strdup(str);
+		}
+	}
+
 }
 
 static gboolean
@@ -1113,6 +1133,7 @@ dissect_sccp_ranap_heur(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
     dissect_ranap(tvb, pinfo, tree);
 
+    
     return TRUE;
 }
 
