@@ -2,7 +2,7 @@
  * Routines for OMA UserPlane Location Protocol packet dissection
  * Copyright 2006, Anders Broman <anders.broman@ericsson.com>
  *
- * $Id: packet-ulp-template.c 19751 2006-10-31 09:29:07Z sahlberg $
+ * $Id: packet-ulp-template.c 21921 2007-05-24 13:30:45Z kukosa $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -34,13 +34,11 @@
 #include <epan/packet.h>
 #include <epan/conversation.h>
 #include <epan/prefs.h>
+#include <epan/asn1.h>
 
 #include <stdio.h>
 #include <string.h>
 
-#include "packet-ulp.h"
-
-#include "packet-ber.h"
 #include "packet-per.h"
 #include <epan/emem.h>
 #include "packet-tcp.h"
@@ -49,8 +47,8 @@
 #define PSNAME "ULP"
 #define PFNAME "ulp"
 
-static dissector_handle_t ulp_handle=NULL;
-static dissector_handle_t rrlp_handle;
+static dissector_handle_t ulp_handle = NULL;
+static dissector_handle_t rrlp_handle = NULL;
 
 /* IANA Registered Ports  
  * oma-ulp         7275/tcp    OMA UserPlane Location
@@ -92,22 +90,8 @@ dissect_ulp_tcp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	tcp_dissect_pdus(tvb, pinfo, tree, ulp_desegment, ULP_HEADER_SIZE,
 	    get_ulp_pdu_len, dissect_ULP_PDU_PDU);
 }
-/*--- proto_reg_handoff_ulp ---------------------------------------*/
-void
-proto_reg_handoff_ulp(void)
-{
-
-	ulp_handle = create_dissector_handle(dissect_ulp_tcp, proto_ulp);
-
-	dissector_add("tcp.port", gbl_ulp_port, ulp_handle);
-
-	/* application/oma-supl-ulp */
-	dissector_add_string("media_type","application/oma-supl-ulp", ulp_handle);
-
-	rrlp_handle = find_dissector("rrlp");
-
-}
 
+void proto_reg_handoff_ulp(void);
 
 /*--- proto_register_ulp -------------------------------------------*/
 void proto_register_ulp(void) {
@@ -151,5 +135,19 @@ void proto_register_ulp(void) {
 }
 
 
+/*--- proto_reg_handoff_ulp ---------------------------------------*/
+void
+proto_reg_handoff_ulp(void)
+{
 
+	ulp_handle = create_dissector_handle(dissect_ulp_tcp, proto_ulp);
+
+	dissector_add("tcp.port", gbl_ulp_port, ulp_handle);
+
+	/* application/oma-supl-ulp */
+	dissector_add_string("media_type","application/oma-supl-ulp", ulp_handle);
+
+	rrlp_handle = find_dissector("rrlp");
+
+}
 
