@@ -1,7 +1,7 @@
 /* column-utils.c
  * Routines for column utilities.
  *
- * $Id: column-utils.c 20118 2006-12-11 20:24:51Z sfisher $
+ * $Id: column-utils.c 21253 2007-03-28 21:55:11Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -622,32 +622,32 @@ col_set_rel_time(frame_data *fd, column_info *cinfo, int col)
 	  case(TS_PREC_FIXED_SEC):
 	  case(TS_PREC_AUTO_SEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->rel_ts.secs, fd->rel_ts.nsecs / 1000000000, SECS);
+			(gint32) fd->rel_ts.secs, fd->rel_ts.nsecs / 1000000000, SECS);
 		  break;
 	  case(TS_PREC_FIXED_DSEC):
 	  case(TS_PREC_AUTO_DSEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->rel_ts.secs, fd->rel_ts.nsecs / 100000000, DSECS);
+			(gint32) fd->rel_ts.secs, fd->rel_ts.nsecs / 100000000, DSECS);
 		  break;
 	  case(TS_PREC_FIXED_CSEC):
 	  case(TS_PREC_AUTO_CSEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->rel_ts.secs, fd->rel_ts.nsecs / 10000000, CSECS);
+			(gint32) fd->rel_ts.secs, fd->rel_ts.nsecs / 10000000, CSECS);
 		  break;
 	  case(TS_PREC_FIXED_MSEC):
 	  case(TS_PREC_AUTO_MSEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->rel_ts.secs, fd->rel_ts.nsecs / 1000000, MSECS);
+			(gint32) fd->rel_ts.secs, fd->rel_ts.nsecs / 1000000, MSECS);
 		  break;
 	  case(TS_PREC_FIXED_USEC):
 	  case(TS_PREC_AUTO_USEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->rel_ts.secs, fd->rel_ts.nsecs / 1000, USECS);
+			(gint32) fd->rel_ts.secs, fd->rel_ts.nsecs / 1000, USECS);
 		  break;
 	  case(TS_PREC_FIXED_NSEC):
 	  case(TS_PREC_AUTO_NSEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->rel_ts.secs, fd->rel_ts.nsecs, NSECS);
+			(gint32) fd->rel_ts.secs, fd->rel_ts.nsecs, NSECS);
 		  break;
 	  default:
 		  g_assert_not_reached();
@@ -666,32 +666,32 @@ col_set_delta_time(frame_data *fd, column_info *cinfo, int col)
 	  case(TS_PREC_FIXED_SEC):
 	  case(TS_PREC_AUTO_SEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->del_ts.secs, fd->del_ts.nsecs / 1000000000, SECS);
+			(gint32) fd->del_cap_ts.secs, fd->del_cap_ts.nsecs / 1000000000, SECS);
 		  break;
 	  case(TS_PREC_FIXED_DSEC):
 	  case(TS_PREC_AUTO_DSEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->del_ts.secs, fd->del_ts.nsecs / 100000000, DSECS);
+			(gint32) fd->del_cap_ts.secs, fd->del_cap_ts.nsecs / 100000000, DSECS);
 		  break;
 	  case(TS_PREC_FIXED_CSEC):
 	  case(TS_PREC_AUTO_CSEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->del_ts.secs, fd->del_ts.nsecs / 10000000, CSECS);
+			(gint32) fd->del_cap_ts.secs, fd->del_cap_ts.nsecs / 10000000, CSECS);
 		  break;
 	  case(TS_PREC_FIXED_MSEC):
 	  case(TS_PREC_AUTO_MSEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->del_ts.secs, fd->del_ts.nsecs / 1000000, MSECS);
+			(gint32) fd->del_cap_ts.secs, fd->del_cap_ts.nsecs / 1000000, MSECS);
 		  break;
 	  case(TS_PREC_FIXED_USEC):
 	  case(TS_PREC_AUTO_USEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->del_ts.secs, fd->del_ts.nsecs / 1000, USECS);
+			(gint32) fd->del_cap_ts.secs, fd->del_cap_ts.nsecs / 1000, USECS);
 		  break;
 	  case(TS_PREC_FIXED_NSEC):
 	  case(TS_PREC_AUTO_NSEC):
 		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
-			fd->del_ts.secs, fd->del_ts.nsecs, NSECS);
+			(gint32) fd->del_cap_ts.secs, fd->del_cap_ts.nsecs, NSECS);
 		  break;
 	  default:
 		  g_assert_not_reached();
@@ -701,6 +701,50 @@ col_set_delta_time(frame_data *fd, column_info *cinfo, int col)
   strcpy(cinfo->col_expr_val[col],cinfo->col_buf[col]);
 }
 
+static void
+col_set_delta_time_dis(frame_data *fd, column_info *cinfo, int col)
+{
+  COL_CHECK_REF_TIME(fd, cinfo, col);
+
+  switch(timestamp_get_precision()) {
+	  case(TS_PREC_FIXED_SEC):
+	  case(TS_PREC_AUTO_SEC):
+		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
+			(gint32) fd->del_dis_ts.secs, fd->del_dis_ts.nsecs / 1000000000, SECS);
+		  break;
+	  case(TS_PREC_FIXED_DSEC):
+	  case(TS_PREC_AUTO_DSEC):
+		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
+			(gint32) fd->del_dis_ts.secs, fd->del_dis_ts.nsecs / 100000000, DSECS);
+		  break;
+	  case(TS_PREC_FIXED_CSEC):
+	  case(TS_PREC_AUTO_CSEC):
+		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
+			(gint32) fd->del_dis_ts.secs, fd->del_dis_ts.nsecs / 10000000, CSECS);
+		  break;
+	  case(TS_PREC_FIXED_MSEC):
+	  case(TS_PREC_AUTO_MSEC):
+		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
+			(gint32) fd->del_dis_ts.secs, fd->del_dis_ts.nsecs / 1000000, MSECS);
+		  break;
+	  case(TS_PREC_FIXED_USEC):
+	  case(TS_PREC_AUTO_USEC):
+		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
+			(gint32) fd->del_dis_ts.secs, fd->del_dis_ts.nsecs / 1000, USECS);
+		  break;
+	  case(TS_PREC_FIXED_NSEC):
+	  case(TS_PREC_AUTO_NSEC):
+		  display_signed_time(cinfo->col_buf[col], COL_MAX_LEN,
+			(gint32) fd->del_dis_ts.secs, fd->del_dis_ts.nsecs, NSECS);
+		  break;
+	  default:
+		  g_assert_not_reached();
+  }
+  cinfo->col_data[col] = cinfo->col_buf[col];
+  strcpy(cinfo->col_expr[col],"frame.time_delta_displayed");
+  strcpy(cinfo->col_expr_val[col],cinfo->col_buf[col]);
+}
+
 /* To do: Add check_col checks to the col_add* routines */
 
 static void
@@ -849,10 +893,16 @@ col_set_cls_time(frame_data *fd, column_info *cinfo, gint col)
     case TS_DELTA:
       col_set_delta_time(fd, cinfo, col);
       break;
+
+    case TS_DELTA_DIS:
+      col_set_delta_time_dis(fd, cinfo, col);
+      break;
+
     case TS_EPOCH:
       col_set_epoch_time(fd, cinfo, col);
       break;
-   case TS_NOT_SET:
+
+    case TS_NOT_SET:
 	/* code is missing for this case, but I don't know which [jmayer20051219] */
 	g_assert(FALSE);
         break;
@@ -1151,6 +1201,10 @@ col_fill_in(packet_info *pinfo)
       col_set_delta_time(pinfo->fd, pinfo->cinfo, i);
       break;
 
+    case COL_DELTA_TIME_DIS:
+      col_set_delta_time_dis(pinfo->fd, pinfo->cinfo, i);
+      break;
+
     case COL_DEF_SRC:
     case COL_RES_SRC:	/* COL_DEF_SRC is currently just like COL_RES_SRC */
       col_set_addr(pinfo, i, &pinfo->src, TRUE, TRUE);
