@@ -1,7 +1,7 @@
 /* column.c
  * Routines for handling column preferences
  *
- * $Id: column.c 20118 2006-12-11 20:24:51Z sfisher $
+ * $Id: column.c 22038 2007-06-04 20:15:10Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -37,6 +37,9 @@
 
 #include <epan/timestamp.h>
 #include <epan/prefs.h>
+#include <epan/nstime.h>
+#include <epan/dfilter/dfilter.h>
+#include "cfile.h"
 #include <epan/column.h>
 #include <epan/packet.h>
 
@@ -51,6 +54,7 @@ col_format_to_string(gint fmt) {
 	"%At",
 	"%Yt",
 	"%Tt",
+	"%Gt",
 	"%s",
 	"%rs",
 	"%us",
@@ -116,6 +120,7 @@ static const gchar *dlist[NUM_COL_FMTS] = {
 	"Absolute time",                            /* COL_ABS_TIME */
 	"Absolute date and time",                   /* COL_ABS_DATE_TIME */
 	"Delta time",                               /* COL_DELTA_TIME */
+	"Delta time displayed",                     /* COL_DELTA_TIME_DIS */
 	"Source address",                           /* COL_DEF_SRC */
 	"Src addr (resolved)",                      /* COL_RES_SRC */
 	"Src addr (unresolved)",                    /* COL_UNRES_SRC */
@@ -355,6 +360,7 @@ get_timestamp_column_longest_string(gint type, gint precision)
 		break;
 	case(TS_RELATIVE):	/* fallthrough */
 	case(TS_DELTA):
+	case(TS_DELTA_DIS):
 		switch(precision) {
 			case(TS_PREC_AUTO_SEC):
 			case(TS_PREC_FIXED_SEC):
@@ -472,6 +478,9 @@ get_column_longest_string(gint format)
     case COL_DELTA_TIME:
       return get_timestamp_column_longest_string(TS_DELTA, timestamp_get_precision());
       break;
+    case COL_DELTA_TIME_DIS:
+      return get_timestamp_column_longest_string(TS_DELTA_DIS, timestamp_get_precision());
+      break;
     case COL_DEF_SRC:
     case COL_RES_SRC:
     case COL_UNRES_SRC:
@@ -581,6 +590,7 @@ get_column_char_width(gint format)
 #define TIME_ABS      2
 #define DATE_TIME_ABS 3
 #define TIME_DEL      4
+#define TIME_DEL_DIS  5
 
 #define RES_DEF  0
 #define RES_DO   1
@@ -672,6 +682,9 @@ get_column_format_from_str(gchar *str) {
       case 'T':
         time_off = TIME_DEL;
         break;
+      case 'G': /* Todo: use a better letter for time since last displayed packet */
+        time_off = TIME_DEL_DIS;
+        break;
       case 'L':
         return COL_PACKET_LENGTH;
         break;
@@ -742,3 +755,46 @@ get_column_title(gint col) {
 
   return(cfmt->title);
 }
+
+void
+build_column_format_array(capture_file *cfile, gboolean reset_fences)
+{
+  int i, j;
+
+  col_setup(&cfile->cinfo, prefs.num_cols);
+
+  for (i = 0; i < cfile->cinfo.num_cols; i++) {
+    cfile->cinfo.col_fmt[i] = get_column_format(i);
+    cfile->cinfo.col_title[i] = g_strdup(get_column_title(i));
+    cfile->cinfo.fmt_matx[i] = (gboolean *) g_malloc0(sizeof(gboolean) *
+						     NUM_COL_FMTS);
+    get_column_format_matches(cfile->cinfo.fmt_matx[i],
+			      cfile->cinfo.col_fmt[i]);
+    cfile->cinfo.col_data[i] = NULL;
+
+    if (cfile->cinfo.col_fmt[i] == COL_INFO)
+      cfile->cinfo.col_buf[i] = (gchar *) g_malloc(sizeof(gchar) *
+						  COL_MAX_INFO_LEN);
+    else
+      cfile->cinfo.col_buf[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
+
+    if(reset_fences)
+      cfile->cinfo.col_fence[i] = 0;
+
+    cfile->cinfo.col_expr[i] = (gchar *) g_malloc(sizeof(gchar) * COL_MAX_LEN);
+    cfile->cinfo.col_expr_val[i] = (gchar *) g_malloc(sizeof(gchar) *
+						     COL_MAX_LEN);
+  }
+    
+  for (i = 0; i < cfile->cinfo.num_cols; i++) {
+    for (j = 0; j < NUM_COL_FMTS; j++) {
+      if (!cfile->cinfo.fmt_matx[i][j])
+	      continue;
+		    
+      if (cfile->cinfo.col_first[j] == -1)
+        cfile->cinfo.col_first[j] = i;
+
+      cfile->cinfo.col_last[j] = i;
+    }
+  }
+}
