@@ -1,7 +1,7 @@
 /* column.h
  * Definitions for column structures and routines
  *
- * $Id: column_info.h 20118 2006-12-11 20:24:51Z sfisher $
+ * $Id: column_info.h 22038 2007-06-04 20:15:10Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -27,6 +27,10 @@
 
 #include <glib.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 #define COL_MAX_LEN 256
 #define COL_MAX_INFO_LEN 4096
 
@@ -43,6 +47,7 @@ typedef struct _column_info {
   gchar      **col_expr;     /* Filter expression */
   gchar      **col_expr_val; /* Value for filter expression */
   gboolean     writable;     /* Are we still writing to the columns? */
+  gboolean columns_changed;  /* Have the columns been changed in the prefs? */
 } column_info;
 
 /*
@@ -58,6 +63,7 @@ enum {
   COL_ABS_TIME,       /* Absolute time */
   COL_ABS_DATE_TIME,  /* Absolute date and time */
   COL_DELTA_TIME,     /* Delta time */
+  COL_DELTA_TIME_DIS, /* Delta time displayed*/
   COL_DEF_SRC,        /* Source address */
   COL_RES_SRC,        /* Resolved source */
   COL_UNRES_SRC,      /* Unresolved source */
@@ -102,13 +108,14 @@ enum {
   COL_8021Q_VLAN_ID,  /* 802.1Q vlan ID */
   COL_DSCP_VALUE,     /* IP DSCP Value */
   COL_COS_VALUE,      /* L2 COS Value */
-  COL_TEI,            /* q.921 TEI */
-  COL_FR_DLCI,		  /* Frame Relay DLCI */
-  COL_BSSGP_TLLI,	  /* GPRS BSSGP IE TLLI */
+  COL_TEI,            /* Q.921 TEI */
+  COL_FR_DLCI,	      /* Frame Relay DLCI */
+  COL_BSSGP_TLLI,     /* GPRS BSSGP IE TLLI */
   NUM_COL_FMTS        /* Should always be last */
 };
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* __COLUMN_INFO_H__ */
-
-
-
