@@ -4,7 +4,7 @@
  * Copyright 2004, Jelmer Vernooij <jelmer@samba.org>
  * Copyright 2004, Devin Heitmueller <dheitmueller@netilla.com>
  *
- * $Id: packet-aim.c 20141 2006-12-17 05:37:18Z sfisher $
+ * $Id: packet-aim.c 21164 2007-03-24 07:33:25Z sfisher $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -542,7 +542,7 @@ const aim_family *aim_get_family( guint16 famnum )
 	return NULL;
 }
 
-int aim_get_buddyname( char *name, tvbuff_t *tvb, int len_offset, int name_offset)
+int aim_get_buddyname( guchar *name, tvbuff_t *tvb, int len_offset, int name_offset)
 {
   guint8 buddyname_length;
 
