@@ -2,7 +2,7 @@
  * Routines for AppleTalk packet disassembly: LLAP, DDP, NBP, ATP, ASP,
  * RTMP, PAP.
  *
- * $Id: packet-atalk.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-atalk.c 21210 2007-03-26 11:06:26Z sahlberg $
  *
  * Simon Wilkinson <sxw@dcs.ed.ac.uk>
  *
@@ -543,13 +543,13 @@ static int dissect_pascal_string(tvbuff_t *tvb, int offset, proto_tree *tree,
 	int hf_index)
 {
 	int len;
+	char *tmp;
 
 	len = tvb_get_guint8(tvb, offset);
 	offset++;
 
 	if ( tree )
 	{
-		char *tmp;
 		proto_tree *item;
 		proto_tree *subtree;
 
@@ -558,7 +558,7 @@ static int dissect_pascal_string(tvbuff_t *tvb, int offset, proto_tree *tree,
 		 * code, we could perhaps avoid allocating and freeing
 		 * this string buffer.
 		 */
-		tmp = tvb_get_ephemeral_string(tvb, offset, len);
+		tmp = (gchar*)tvb_get_ephemeral_string(tvb, offset, len);
 		item = proto_tree_add_string(tree, hf_index, tvb, offset-1, len+1, tmp);
 
 		subtree = proto_item_add_subtree(item, ett_pstring);
@@ -980,6 +980,15 @@ dissect_asp_reply_get_status(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *
 	guint8  len;
 	guint8  i;
 
+  	proto_tree *adr_tree;
+  	char *tmp;
+  	const guint8 *ip;
+  	guint16 net;
+  	guint8  node;
+  	guint16 port;
+
+	guint16 ulen;
+
 	if (!tree)
 		return offset;
 
@@ -1087,12 +1096,6 @@ dissect_asp_reply_get_status(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *
 	}
 
 	if (adr_ofs) {
-        	proto_tree *adr_tree;
-		char *tmp;
-        	const guint8 *ip;
-		guint16 net;
-		guint8  node;
-        	guint16 port;
 
 		ofs = adr_ofs;
 		nbe = tvb_get_guint8(tvb, ofs);
@@ -1128,7 +1131,7 @@ dissect_asp_reply_get_status(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *
 				break;
 			case 4: /* DNS */
 				if (len > 2) {
-					tmp = tvb_get_ephemeral_string(tvb, ofs +2, len -2);
+					tmp = (gchar*)tvb_get_ephemeral_string(tvb, ofs +2, len -2);
 					ti = proto_tree_add_text(adr_tree, tvb, ofs, len, "dns %s", tmp);
 					break;
 				}
@@ -1161,12 +1164,10 @@ dissect_asp_reply_get_status(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *
 		}
 	}
 	if (utf_ofs) {
-		guint16 ulen;
-		char *tmp;
 
 		ofs = utf_ofs;
 		ulen = tvb_get_ntohs(tvb, ofs);
-		tmp = tvb_get_ephemeral_string(tvb, ofs + 2, ulen);
+		tmp = (gchar*)tvb_get_ephemeral_string(tvb, ofs + 2, ulen);
 		ti = proto_tree_add_text(tree, tvb, ofs, ulen +2, "UTF8 server name: %s", tmp);
 		sub_tree = proto_item_add_subtree(ti, ett_asp_utf8_name);
 		proto_tree_add_uint(sub_tree, hf_asp_server_utf8_name_len, tvb, ofs, 2, ulen);
