@@ -1,7 +1,7 @@
 /* packet-beep.c
  * Routines for BEEP packet disassembly
  *
- * $Id: packet-beep.c 18196 2006-05-21 04:49:01Z sahlberg $
+ * $Id: packet-beep.c 21210 2007-03-26 11:06:26Z sahlberg $
  *
  * Copyright (c) 2000 by Richard Sharpe <rsharpe@ns.aus.com>
  * Modified 2001 Darren New <dnew@invisible.net> for BEEP.
@@ -447,7 +447,7 @@ dissect_beep_int(tvbuff_t *tvb, int offset,
 
   /* XXX - is this still "Dangerous" now that we don't copy to the
      last byte of "int_buff[]"? */
-  sscanf(int_buff, "%d", &ival);
+  sscanf((gchar*)int_buff, "%d", &ival);
 
   if (tree) {
     proto_tree_add_uint(tree, hf, tvb, offset, i, ival);
