@@ -2,7 +2,7 @@
  * Routines for FC Extended Link Services
  * Copyright 2001, Dinesh G Dutt <ddutt@cisco.com>
  *
- * $Id: packet-fcels.c 19492 2006-10-11 10:38:59Z sahlberg $
+ * $Id: packet-fcels.c 22088 2007-06-13 06:54:49Z jake $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -1924,7 +1924,12 @@ dissect_fcels (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     /* Register conversation in case this is not a response */
     if ((opcode != FC_ELS_LSRJT) && (opcode != FC_ELS_ACC)) {
         if (opcode == FC_ELS_FLOGI) {
-            if (pinfo->src.data[2]) {
+            const guint8 *srcfc;
+
+            /* Check that the source address is, in fact, an FC address */
+            DISSECTOR_ASSERT(pinfo->src.type == AT_FC);
+            srcfc = pinfo->src.data;
+            if (srcfc[2]) {
                 /* If it is a loop port, we'll need to remember the ALPA */
                 options = NO_PORT2;
             }
@@ -1977,8 +1982,14 @@ dissect_fcels (tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
             /* FLOGI has two ways to save state: without the src and using just
              * the port (ALPA) part of the address. Try both.
              */
+            const guint8 *dstfc;
+
+            /* Check that the source address is, in fact, an FC address */
+            DISSECTOR_ASSERT(pinfo->dst.type == AT_FC);
+            dstfc = pinfo->dst.data;
+
             addrdata[0] = addrdata[1] = 0;
-            addrdata[2] = pinfo->dst.data[2];
+            addrdata[2] = dstfc[2];
             SET_ADDRESS (&dstaddr, AT_FC, 3, addrdata);
             conversation = find_conversation (pinfo->fd->num, &dstaddr, &pinfo->src,
                                               pinfo->ptype, pinfo->oxid,
@@ -2466,13 +2477,13 @@ proto_register_fcels (void)
            HFILL}},
         { &hf_fcels_fcpflags_trireq,
           {"Task Retry Ident", "fcels.fcpflags.trireq", FT_BOOLEAN, 32,
-           TFS(&tfs_fc_fcels_fcpflags_trireq), 0x2000, "", HFILL}},
+           TFS(&tfs_fc_fcels_fcpflags_trireq), 1 << 9, "", HFILL}},
         { &hf_fcels_fcpflags_trirep,
           {"Task Retry Ident", "fcels.fcpflags.trirep", FT_BOOLEAN, 32,
-           TFS(&tfs_fc_fcels_fcpflags_trirep), 0x2000, "", HFILL}},
+           TFS(&tfs_fc_fcels_fcpflags_trirep), 1 << 9, "", HFILL}},
         { &hf_fcels_fcpflags_retry,
           {"Retry", "fcels.fcpflags.retry", FT_BOOLEAN, 32,
-           TFS(&tfs_fc_fcels_fcpflags_retry), 0x1000, "", HFILL}},
+           TFS(&tfs_fc_fcels_fcpflags_retry), 1 << 8, "", HFILL}},
         { &hf_fcels_fcpflags_ccomp,
           {"Comp", "fcels.fcpflags.ccomp", FT_BOOLEAN, 32,
            TFS(&tfs_fc_fcels_fcpflags_ccomp), 0x0080, "", HFILL}},
