@@ -2,7 +2,7 @@
  *
  * Top-most dissector. Decides dissector based on Wiretap Encapsulation Type.
  *
- * $Id: packet-frame.c 20437 2007-01-15 05:16:13Z ulfl $
+ * $Id: packet-frame.c 22094 2007-06-13 22:36:58Z gerald $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -43,9 +43,11 @@ int proto_frame = -1;
 int hf_frame_arrival_time = -1;
 static int hf_frame_time_invalid = -1;
 static int hf_frame_time_delta = -1;
+static int hf_frame_time_delta_displayed = -1;
 static int hf_frame_time_relative = -1;
 int hf_frame_number = -1;
-int hf_frame_packet_len = -1;
+int hf_frame_packet_len = -1; /* Deprecated in favor of hf_frame_len */
+int hf_frame_len = -1;
 int hf_frame_capture_len = -1;
 static int hf_frame_p2p_dir = -1;
 static int hf_frame_file_off = -1;
@@ -77,11 +79,11 @@ static const value_string p2p_dirs[] = {
 	{ 0, NULL }
 };
 
-static dissector_table_t wtap_encap_dissector_table;
+dissector_table_t wtap_encap_dissector_table;
 
 static GSList *frame_end_routines = NULL;
 
-/* 
+/*
  * Routine used to register frame end routine.  The routine should only
  * be registred when the dissector is used in the frame, not in the
  * proto_register_XXX function.
@@ -107,10 +109,10 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	proto_tree	*fh_tree=NULL;
 	proto_item	*volatile ti = NULL;
 	nstime_t	ts;
-	int		cap_len = 0, pkt_len = 0;
-	proto_tree	*tree;
+	int		cap_len = 0, frame_len = 0;
+	proto_tree	*volatile tree;
         proto_item  *item;
-	int frame_number;
+	guint32 frame_number;
 
 	frame_number=pinfo->fd->num; /* dummy so that the buildbot crashdumps
 					will show the packetnumber where the
@@ -154,20 +156,19 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 			pinfo->p2p_dir = pinfo->pseudo_header->mtp2.sent ?
 			    P2P_DIR_SENT : P2P_DIR_RECV;
 			pinfo->link_number  = pinfo->pseudo_header->mtp2.link_number;
-			pinfo->annex_a_used = pinfo->pseudo_header->mtp2.annex_a_used ?
-			    MTP2_ANNEX_A_USED : MTP2_ANNEX_A_NOT_USED;
+			pinfo->annex_a_used = pinfo->pseudo_header->mtp2.annex_a_used;
 			break;
-		
+
 		}
 	}
 
 	/* Put in frame header information. */
 	if (tree) {
 	  cap_len = tvb_length(tvb);
-	  pkt_len = tvb_reported_length(tvb);
+	  frame_len = tvb_reported_length(tvb);
 
 	  ti = proto_tree_add_protocol_format(tree, proto_frame, tvb, 0, -1,
-	    "Frame %u (%u bytes on wire, %u bytes captured)", pinfo->fd->num, pkt_len, cap_len);
+	    "Frame %u (%u bytes on wire, %u bytes captured)", pinfo->fd->num, frame_len, cap_len);
 
 	  fh_tree = proto_item_add_subtree(ti, ett_frame);
 	}
@@ -181,7 +182,7 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		fh_tree = NULL;
 	}
 
-       
+
 	if (fh_tree) {
 	  ts = pinfo->fd->abs_ts;
 
@@ -194,12 +195,18 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 		expert_add_info_format(pinfo, item, PI_MALFORMED, PI_WARN, "Arrival Time: Fractional second out of range (0-1000000000)");
       }
 
-	  ts = pinfo->fd->del_ts;
+	  ts = pinfo->fd->del_cap_ts;
 
 	  item = proto_tree_add_time(fh_tree, hf_frame_time_delta, tvb,
 		0, 0, &ts);
 	  PROTO_ITEM_SET_GENERATED(item);
 
+	  ts = pinfo->fd->del_dis_ts;
+
+	  item = proto_tree_add_time(fh_tree, hf_frame_time_delta_displayed, tvb,
+		0, 0, &ts);
+	  PROTO_ITEM_SET_GENERATED(item);
+
 	  ts = pinfo->fd->rel_ts;
 
 	  item = proto_tree_add_time(fh_tree, hf_frame_time_relative, tvb,
@@ -209,9 +216,15 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	  proto_tree_add_uint(fh_tree, hf_frame_number, tvb,
 		0, 0, pinfo->fd->num);
 
-	  proto_tree_add_uint_format(fh_tree, hf_frame_packet_len, tvb,
-		0, 0, pkt_len, "Packet Length: %d byte%s", pkt_len,
-		plurality(pkt_len, "", "s"));
+	  /* Deprecated in favor of hf_frame_len */
+	  item = proto_tree_add_uint_format(fh_tree, hf_frame_packet_len, tvb,
+		0, 0, frame_len, "Packet Length: %d byte%s", frame_len,
+		plurality(frame_len, "", "s"));
+	  PROTO_ITEM_SET_HIDDEN(item);
+
+	  proto_tree_add_uint_format(fh_tree, hf_frame_len, tvb,
+		0, 0, frame_len, "Frame Length: %d byte%s", frame_len,
+		plurality(frame_len, "", "s"));
 
 	  proto_tree_add_uint_format(fh_tree, hf_frame_capture_len, tvb,
 		0, 0, cap_len, "Capture Length: %d byte%s", cap_len,
@@ -254,7 +267,7 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	  if (show_file_off) {
 		  proto_tree_add_int64_format(fh_tree, hf_frame_file_off, tvb,
 				  0, 0, pinfo->fd->file_off,
-				  "File Offset: %lld (0x%llx)",
+				  "File Offset: %" G_GINT64_MODIFIER "d (0x%" G_GINT64_MODIFIER "x)",
 				  pinfo->fd->file_off, pinfo->fd->file_off);
 	  }
 
@@ -298,17 +311,22 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
     } __except(TRUE /* handle all exceptions */) {
         switch(GetExceptionCode()) {
         case(STATUS_ACCESS_VIOLATION):
-		    show_exception(tvb, pinfo, parent_tree, DissectorError, 
+		    show_exception(tvb, pinfo, parent_tree, DissectorError,
                 "STATUS_ACCESS_VIOLATION: dissector accessed an invalid memory address");
             break;
         case(STATUS_INTEGER_DIVIDE_BY_ZERO):
 		    show_exception(tvb, pinfo, parent_tree, DissectorError,
                 "STATUS_INTEGER_DIVIDE_BY_ZERO: dissector tried an integer division by zero");
             break;
+        case(STATUS_STACK_OVERFLOW):
+		    show_exception(tvb, pinfo, parent_tree, DissectorError,
+                "STATUS_STACK_OVERFLOW: dissector overflowed the stack (e.g. endless loop)");
+            /* XXX - this will have probably corrupted the stack, which makes problems later in the exception code */
+            break;
         /* XXX - add other hardware exception codes as required */
         default:
-		    show_exception(tvb, pinfo, parent_tree, DissectorError, 
-                g_strdup_printf("dissector caused an unknown exception: %u", GetExceptionCode()));
+		    show_exception(tvb, pinfo, parent_tree, DissectorError,
+                g_strdup_printf("dissector caused an unknown exception: 0x%x", GetExceptionCode()));
         }
     }
 #endif
@@ -321,16 +339,16 @@ dissect_frame(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tree)
 	}
 	ENDTRY;
 
-	if (tree) {
+	if (tree && pinfo->layer_names) {
 		proto_item_append_string(ti, pinfo->layer_names->str);
 		g_string_free(pinfo->layer_names, TRUE);
 		pinfo->layer_names = NULL;
 	}
 
-    call_all_postdissectors(tvb,pinfo,parent_tree);
+	call_all_postdissectors(tvb, pinfo, parent_tree);
 
 	tap_queue_packet(frame_tap, pinfo, NULL);
-    
+
 
 	if (frame_end_routines) {
 		g_slist_foreach(frame_end_routines, &call_frame_end_routine, NULL);
@@ -367,8 +385,8 @@ show_exception(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 		/*item =*/ proto_tree_add_protocol_format(tree, proto_short, tvb, 0, 0,
 				"[Packet size limited during capture: %s truncated]", pinfo->current_proto);
 		/* Don't record BoundsError exceptions as expert events - they merely
-		 * reflect a capture done with a snapshot length too short to capture 
-		 * all of the packet 
+		 * reflect a capture done with a snapshot length too short to capture
+		 * all of the packet
 		 * (any case where it's caused by something else is a bug). */
 		/* expert_add_info_format(pinfo, item, PI_MALFORMED, PI_ERROR, "Packet size limited");*/
 		break;
@@ -445,14 +463,19 @@ proto_register_frame(void)
 		{ "Arrival Time",		"frame.time", FT_ABSOLUTE_TIME, BASE_NONE, NULL, 0x0,
 			"Absolute time when this frame was captured", HFILL }},
 
-        { &hf_frame_time_invalid,
+		{ &hf_frame_time_invalid,
 		{ "Arrival Timestamp invalid",		"frame.time_invalid", FT_NONE, BASE_NONE, NULL, 0x0,
 			"The timestamp from the capture is out of the valid range", HFILL }},
 
 		{ &hf_frame_time_delta,
-		{ "Time delta from previous packet",	"frame.time_delta", FT_RELATIVE_TIME, BASE_NONE, NULL,
+		{ "Time delta from previous captured frame",	"frame.time_delta", FT_RELATIVE_TIME, BASE_NONE, NULL,
 			0x0,
-			"Time delta since previous displayed frame", HFILL }},
+			"Time delta from previous captured frame", HFILL }},
+
+		{ &hf_frame_time_delta_displayed,
+		{ "Time delta from previous displayed frame",	"frame.time_delta_displayed", FT_RELATIVE_TIME, BASE_NONE, NULL,
+			0x0,
+			"Time delta from previous displayed frame", HFILL }},
 
 		{ &hf_frame_time_relative,
 		{ "Time since reference or first frame",	"frame.time_relative", FT_RELATIVE_TIME, BASE_NONE, NULL,
@@ -463,10 +486,15 @@ proto_register_frame(void)
 		{ "Frame Number",		"frame.number", FT_UINT32, BASE_DEC, NULL, 0x0,
 			"", HFILL }},
 
+		/* Deprecated and hidden in favor of hf_frame_len / frame.len */
 		{ &hf_frame_packet_len,
 		{ "Frame length on the wire",		"frame.pkt_len", FT_UINT32, BASE_DEC, NULL, 0x0,
 			"", HFILL }},
 
+		{ &hf_frame_len,
+		{ "Frame length on the wire",		"frame.len", FT_UINT32, BASE_DEC, NULL, 0x0,
+			"", HFILL }},
+
 		{ &hf_frame_capture_len,
 		{ "Frame length stored into the capture file",	"frame.cap_len", FT_UINT32, BASE_DEC, NULL, 0x0,
 			"", HFILL }},
@@ -500,10 +528,10 @@ proto_register_frame(void)
 			"The frame matched the coloring rule with this name", HFILL }},
 		{ &hf_frame_color_filter_text,
 		{ "Coloring Rule String", "frame.coloring_rule.string", FT_STRING, 0, NULL, 0x0,
-			"The frame matched this coloring rule string", HFILL }},
+			"The frame matched this coloring rule string", HFILL }}
     };
 	static gint *ett[] = {
-		&ett_frame,
+		&ett_frame
 	};
 	module_t *frame_module;
 
