@@ -4,7 +4,7 @@
  * Copyright 2002, Richard Sharpe <rsharpe@samba.org> Added a few 
  *		   bits and pieces ...
  *
- * $Id: packet-gssapi.c 20136 2006-12-14 10:11:40Z sahlberg $
+ * $Id: packet-gssapi.c 21773 2007-05-15 05:49:43Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -45,6 +45,7 @@
 #include "epan/emem.h"
 #include "epan/prefs.h"
 #include "epan/reassemble.h"
+#include <epan/asn1.h>
 #include "packet-ber.h"
 #include "to_str.h"
 
@@ -172,13 +173,14 @@ dissect_gssapi_work(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 {
 	proto_item *item;
 	proto_tree *subtree;
-	int return_offset = 0;
-	gssapi_conv_info_t *gss_info;
+	volatile int return_offset = 0;
+	gssapi_conv_info_t *volatile gss_info;
 	gssapi_oid_value *oidvalue;
 	dissector_handle_t handle;
 	conversation_t *conversation;
 	tvbuff_t *oid_tvb;
-	int len, offset, start_offset, oid_start_offset;
+	int len, start_offset, oid_start_offset;
+	volatile int offset;
 	gint8 class;
 	gboolean pc, ind_field;
 	gint32 tag;
@@ -186,11 +188,12 @@ dissect_gssapi_work(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	const char *oid;
 	fragment_data *fd_head=NULL;
 	gssapi_frag_info_t *fi;
-	tvbuff_t *gss_tvb=NULL;
+	tvbuff_t *volatile gss_tvb=NULL;
+	asn1_ctx_t asn1_ctx;
 
 	start_offset=0;
 	offset=0;
-
+	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
 	/*
 	 * We don't know whether the data is encrypted, so say it's
 	 * not, for now.  The subdissector must set gssapi_data_encrypted
@@ -365,7 +368,7 @@ dissect_gssapi_work(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 
 		/* Read oid */
 		oid_start_offset=offset;
-		offset=dissect_ber_object_identifier_str(FALSE, pinfo, subtree, gss_tvb, offset, hf_gssapi_oid, &oid);
+		offset=dissect_ber_object_identifier_str(FALSE, &asn1_ctx, subtree, gss_tvb, offset, hf_gssapi_oid, &oid);
 		oidvalue = gssapi_lookup_oid_str(oid);
 
 
