@@ -4,7 +4,7 @@
  *
  *  (c) 2006, Luis E. Garcia Ontanon <luis.ontanon@gmail.com>
  *
- * $Id: packet-h248_annex_c.c 20272 2007-01-02 17:19:25Z wmeier $
+ * $Id: packet-h248_annex_c.c 21773 2007-05-15 05:49:43Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -32,7 +32,6 @@
 
 /* H.248 Annex C */
 static int proto_h248_pkg_annexc = -1;
-static int hf_h248_pkg_annexc_parameters = -1;
 
 static int hf_h248_pkg_annexc_media = -1;
 static int hf_h248_pkg_annexc_ACodec = -1;
@@ -176,14 +175,6 @@ static gint ett_annexc = -1;
 static gint ett_vpvc = -1;
 static gint ett_codec = -1;
 
-static int two = 2;
-static int three = 3;
-static int four = 4;
-static int twelve = 12;
-static int sixteen = 16;
-static int twenty = 20;
-static int thirty = 30;
-
 static value_string h248_annexc_package_properties_vals[] = {
 	{ 0x1001, "media" },
 	{ 0x1002, "transmission mode" },
@@ -347,7 +338,7 @@ static const value_string h248_annexc_transmission_mode[] = {
     {0,NULL}
 };
 
-static const value_string h248_pkg_annexc_sc[] = {
+static const value_string h248_pkg_annexc_sc[] _U_ = {
     {0,"CBR"},
     {1,"nrt-VBR1"},
     {2,"nrt-VBR2"},
@@ -454,7 +445,7 @@ static const value_string h248_pkg_annexc_nicrx_values[] = {
 };
 
 
-static const value_string h248_pkg_annexc_flowconttx_values[] = {
+static const value_string h248_pkg_annexc_flowconttx_values[] _U_ = {
 	{0,"Not Required"},
 	{1,"Required"},
     {0,NULL}
@@ -617,7 +608,7 @@ static const value_string h248_pkg_annexc_addlayer3prot_values[] = {
     {0,NULL}
 };
 
-static const value_string h248_pkg_annexc_nci_satelite_values[] = {
+static const value_string h248_pkg_annexc_nci_satelite_values[] _U_ = {
 	{0x0,"no satelite circuit"},
 	{0x1,"one satellite circuit"},
 	{0x2,"two satelite circiuts"},
@@ -625,7 +616,7 @@ static const value_string h248_pkg_annexc_nci_satelite_values[] = {
     {0,NULL}
 };
 
-static const value_string h248_pkg_annexc_nci_continuity_values[] = {
+static const value_string h248_pkg_annexc_nci_continuity_values[] _U_ = {
 	{0x0,"continuity check not required"},
 	{0x1,"continuity check required on this circuit"},
 	{0x2,"continuity check performed on a previous circuit"},
@@ -633,7 +624,7 @@ static const value_string h248_pkg_annexc_nci_continuity_values[] = {
     {0,NULL}
 };
 
-static const value_string h248_pkg_annexc_nci_echoctl_values[] = {
+static const value_string h248_pkg_annexc_nci_echoctl_values[] _U_ = {
 	{0x0,"outgoing echo control device not included"},
 	{0x1,"outgoing echo control device included"},
     {0,NULL}
@@ -746,8 +737,10 @@ static void dissect_h248_annexc_acodec(proto_tree* tree,
 										void* implicit_p ) {
 	int len;
 	tvbuff_t* new_tvb;
-	
-	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, pinfo, tree, tvb, 0, hfid, &new_tvb);
+	asn1_ctx_t asn1_ctx;
+
+	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
+	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, &asn1_ctx, tree, tvb, 0, hfid, &new_tvb);
 	
 	tree = proto_item_add_subtree(get_ber_last_created_item(),ett_codec);
 	len = tvb_length(new_tvb);
@@ -761,8 +754,10 @@ static void dissect_h248_annexc_BIR(proto_tree* tree,
 									 h248_curr_info_t* h248_info,
 									 void* implicit_p ) {
 	tvbuff_t* new_tvb = NULL;
-	
-	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, pinfo, tree, tvb, 0, hfid, &new_tvb);
+	asn1_ctx_t asn1_ctx;
+
+	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);	
+	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, &asn1_ctx, tree, tvb, 0, hfid, &new_tvb);
 	
 	if ( new_tvb && h248_info->term && ! h248_info->term->bir ) {
 		h248_info->term->bir = se_strdup(tvb_bytes_to_str(new_tvb,0,tvb_length(new_tvb)));
@@ -776,7 +771,10 @@ static void dissect_h248_annexc_NSAP(proto_tree* tree,
 									  h248_curr_info_t* h248_info,
 									  void* implicit_p ) {
 	tvbuff_t* new_tvb = NULL;
-	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, pinfo, tree, tvb, 0, hfid, &new_tvb);
+	asn1_ctx_t asn1_ctx;
+
+	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
+	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, &asn1_ctx, tree, tvb, 0, hfid, &new_tvb);
 	if (new_tvb) {
 		dissect_nsap(new_tvb, 0,tvb_length_remaining(new_tvb, 0), tree);
 		if ( h248_info->term && ! h248_info->term->nsap) {
@@ -792,15 +790,14 @@ static void dissect_h248_annexc_vpvc(proto_tree* tree, tvbuff_t* tvb, packet_inf
 	proto_tree_add_item(pt,hf_h248_pkg_annexc_vc,tvb,2,2,FALSE);
 }
 
-static void dissect_byte_param(proto_tree* tree, tvbuff_t* tvb, packet_info* pinfo _U_, int hfid, h248_curr_info_t* h248_info _U_, void* unused _U_) {
-	proto_tree_add_item(tree,hfid,tvb,0,1,FALSE);
-}
-
 static void dissect_h248_annexc_USI(proto_tree* tree, tvbuff_t* tvb, packet_info* pinfo, int hfid, h248_curr_info_t* h248_info _U_, void* implicit_p) {
 	tvbuff_t* new_tvb = NULL;
-	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, pinfo, tree, tvb, 0, hfid, &new_tvb);
+	asn1_ctx_t asn1_ctx;
+
+	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
+	dissect_ber_octet_string(implicit_p ? *((gboolean*)implicit_p) : FALSE, &asn1_ctx, tree, tvb, 0, hfid, &new_tvb);
 	if (new_tvb)
-		dissect_q931_bearer_capability_ie(new_tvb, 0, 3, tree);
+		dissect_q931_bearer_capability_ie(new_tvb, 0, tvb_length(new_tvb), tree);
 }
 
 gboolean h248_c_implicit = TRUE;
@@ -830,71 +827,71 @@ static h248_pkg_param_t h248_annexc_package_properties[] = {
 	{ 0x3002, &hf_h248_pkg_annexc_BIR, dissect_h248_annexc_BIR, NULL },
 	{ 0x3003, &hf_h248_pkg_annexc_NSAP, dissect_h248_annexc_NSAP, NULL },
 	
-	{ 0x4001, &hf_h248_pkg_annexc_aesa, h248_param_item, &twenty },
+	{ 0x4001, &hf_h248_pkg_annexc_aesa, h248_param_ber_octetstring, NULL },
 	{ 0x4002, &hf_h248_pkg_annexc_vp, dissect_h248_annexc_vpvc, NULL },
 	{ 0x4003, &hf_h248_pkg_annexc_sc, h248_param_ber_integer, NULL },
-	{ 0x4004, &hf_h248_pkg_annexc_bcob, dissect_byte_param, NULL },
-	{ 0x4005, &hf_h248_pkg_annexc_bbtc, dissect_byte_param, NULL },
+	{ 0x4004, &hf_h248_pkg_annexc_bcob, h248_param_ber_integer, NULL },
+	{ 0x4005, &hf_h248_pkg_annexc_bbtc, h248_param_ber_integer, NULL },
 	{ 0x4006, &hf_h248_pkg_annexc_atc, h248_param_ber_integer, NULL },
-	{ 0x4007, &hf_h248_pkg_annexc_stc, dissect_byte_param, NULL },
-	{ 0x4008, &hf_h248_pkg_annexc_uppc, dissect_byte_param, NULL },
-	{ 0x4009, &hf_h248_pkg_annexc_pcr0, h248_param_item, &three },
-	{ 0x400a, &hf_h248_pkg_annexc_scr0, h248_param_item, &three },
-	{ 0x400b, &hf_h248_pkg_annexc_mbs0, h248_param_item, &three },
-	{ 0x400c, &hf_h248_pkg_annexc_pcr1, h248_param_item, &three },
-	{ 0x400d, &hf_h248_pkg_annexc_scr1, h248_param_item, &three },
-	{ 0x400e, &hf_h248_pkg_annexc_mbs1, h248_param_item, &three },
+	{ 0x4007, &hf_h248_pkg_annexc_stc, h248_param_ber_integer, NULL },
+	{ 0x4008, &hf_h248_pkg_annexc_uppc, h248_param_ber_integer, NULL },
+	{ 0x4009, &hf_h248_pkg_annexc_pcr0, h248_param_ber_octetstring, NULL },
+	{ 0x400a, &hf_h248_pkg_annexc_scr0, h248_param_ber_octetstring, NULL },
+	{ 0x400b, &hf_h248_pkg_annexc_mbs0, h248_param_ber_octetstring, NULL },
+	{ 0x400c, &hf_h248_pkg_annexc_pcr1, h248_param_ber_octetstring, NULL },
+	{ 0x400d, &hf_h248_pkg_annexc_scr1, h248_param_ber_octetstring, NULL },
+	{ 0x400e, &hf_h248_pkg_annexc_mbs1, h248_param_ber_octetstring, NULL },
 	{ 0x400f, &hf_h248_pkg_annexc_bei, h248_param_ber_boolean, NULL },
 	{ 0x4010, &hf_h248_pkg_annexc_ti, h248_param_ber_boolean, NULL },
 	{ 0x4011, &hf_h248_pkg_annexc_fd, h248_param_ber_boolean, NULL },
-	{ 0x4012, &hf_h248_pkg_annexc_a2pcdv, h248_param_item, &three },
-	{ 0x4013, &hf_h248_pkg_annexc_c2pcdv, h248_param_item, &three },
-	{ 0x4014, &hf_h248_pkg_annexc_appcdv, h248_param_item, &three },
-	{ 0x4015, &hf_h248_pkg_annexc_cppcdv, h248_param_item, &three },
-	{ 0x4016, &hf_h248_pkg_annexc_aclr, dissect_byte_param, NULL },
-	{ 0x4017, &hf_h248_pkg_annexc_meetd, h248_param_item, &two },
-	{ 0x4018, &hf_h248_pkg_annexc_ceetd, h248_param_item, &two },
+	{ 0x4012, &hf_h248_pkg_annexc_a2pcdv, h248_param_ber_octetstring, NULL },
+	{ 0x4013, &hf_h248_pkg_annexc_c2pcdv, h248_param_ber_octetstring, NULL },
+	{ 0x4014, &hf_h248_pkg_annexc_appcdv, h248_param_ber_octetstring, NULL },
+	{ 0x4015, &hf_h248_pkg_annexc_cppcdv, h248_param_ber_octetstring, NULL },
+	{ 0x4016, &hf_h248_pkg_annexc_aclr, h248_param_ber_integer, NULL },
+	{ 0x4017, &hf_h248_pkg_annexc_meetd, h248_param_ber_octetstring,NULL },
+	{ 0x4018, &hf_h248_pkg_annexc_ceetd, h248_param_ber_octetstring,NULL },
 	{ 0x4019, &hf_h248_pkg_annexc_QosClass, h248_param_ber_integer, NULL },
-	{ 0x401A, &hf_h248_pkg_annexc_AALtype, dissect_byte_param, NULL },
+	{ 0x401A, &hf_h248_pkg_annexc_AALtype, h248_param_ber_integer, NULL },
 
 	{ 0x5001, &hf_h248_pkg_annexc_dlci, h248_param_ber_integer, NULL },
 	{ 0x5002, &hf_h248_pkg_annexc_cid, h248_param_ber_integer, NULL },
 	{ 0x5003, &hf_h248_pkg_annexc_sid, h248_param_ber_integer, NULL },
 	{ 0x5004, &hf_h248_pkg_annexc_ppt, h248_param_ber_integer, NULL },
 	
-	{ 0x6001, &hf_h248_pkg_annexc_ipv4, h248_param_item, &four },
-	{ 0x6002, &hf_h248_pkg_annexc_ipv6, h248_param_item, &sixteen },
+	{ 0x6001, &hf_h248_pkg_annexc_ipv4, h248_param_ber_octetstring, NULL },
+	{ 0x6002, &hf_h248_pkg_annexc_ipv6, h248_param_ber_octetstring,NULL },
 	{ 0x6003, &hf_h248_pkg_annexc_port, h248_param_ber_integer, NULL },
 	{ 0x6004, &hf_h248_pkg_annexc_porttype, h248_param_ber_integer, NULL },
 	
-	{ 0x7001, &hf_h248_pkg_annexc_aesa, h248_param_item, &twenty },
-	{ 0x7002, &hf_h248_pkg_annexc_alc, h248_param_item, &twelve }, /* from ALCAP */
-	{ 0x7003, &hf_h248_pkg_annexc_sscs, h248_param_item, NULL }, 
-	{ 0x7004, &hf_h248_pkg_annexc_sut, h248_param_item, NULL }, 
+	{ 0x7001, &hf_h248_pkg_annexc_aesa, h248_param_ber_octetstring, NULL },
+	{ 0x7002, &hf_h248_pkg_annexc_alc, h248_param_ber_octetstring,NULL }, /* from ALCAP */
+	{ 0x7003, &hf_h248_pkg_annexc_sscs, h248_param_ber_octetstring, NULL }, 
+	{ 0x7004, &hf_h248_pkg_annexc_sut, h248_param_ber_octetstring, NULL }, 
 	{ 0x7005, &hf_h248_pkg_annexc_tci, h248_param_ber_boolean, NULL }, 
-	{ 0x7006, &hf_h248_pkg_annexc_timer_cu, h248_param_item, &four }, 
-	{ 0x7007, &hf_h248_pkg_annexc_maxcpssdu, dissect_byte_param, NULL }, 
-	{ 0x7008, &hf_h248_pkg_annexc_cid, dissect_byte_param, NULL }, 
+	{ 0x7006, &hf_h248_pkg_annexc_timer_cu, h248_param_ber_octetstring, NULL }, 
+	{ 0x7007, &hf_h248_pkg_annexc_maxcpssdu, h248_param_ber_integer, NULL }, 
+	{ 0x7008, &hf_h248_pkg_annexc_cid, h248_param_ber_integer, NULL }, 
 	
-	{ 0x8001, &hf_h248_pkg_annexc_aal1st, dissect_byte_param, NULL },
-	{ 0x8002, &hf_h248_pkg_annexc_cbrr, dissect_byte_param, NULL },
-	{ 0x8003, &hf_h248_pkg_annexc_scri, dissect_byte_param, NULL }, 
-	{ 0x8004, &hf_h248_pkg_annexc_ecm, dissect_byte_param, NULL }, 
-	{ 0x8005, &hf_h248_pkg_annexc_sdbt, h248_param_item, &two }, 
-	{ 0x8006, &hf_h248_pkg_annexc_pfci, dissect_byte_param, NULL }, 
+	{ 0x8001, &hf_h248_pkg_annexc_aal1st, h248_param_ber_integer, NULL },
+	{ 0x8002, &hf_h248_pkg_annexc_cbrr, h248_param_ber_integer, NULL },
+	{ 0x8003, &hf_h248_pkg_annexc_scri, h248_param_ber_integer, NULL }, 
+	{ 0x8004, &hf_h248_pkg_annexc_ecm, h248_param_ber_integer, NULL }, 
+	{ 0x8005, &hf_h248_pkg_annexc_sdbt, h248_param_ber_octetstring,NULL }, 
+	{ 0x8006, &hf_h248_pkg_annexc_pfci, h248_param_ber_integer, NULL }, 
 
-	{ 0x9001, &hf_h248_pkg_annexc_tmr, dissect_byte_param, NULL },
-	{ 0x9002, &hf_h248_pkg_annexc_tmrsr, dissect_byte_param, NULL },
-	{ 0x9003, &hf_h248_pkg_annexc_contcheck, dissect_byte_param, NULL },
-	{ 0x9004, &hf_h248_pkg_annexc_itc, dissect_byte_param, NULL },
-	{ 0x9005, &hf_h248_pkg_annexc_transmode, dissect_byte_param, NULL },
-	{ 0x9006, &hf_h248_pkg_annexc_transrate, dissect_byte_param, NULL },
-	{ 0x9007, &hf_h248_pkg_annexc_mult, dissect_byte_param, NULL },
+	{ 0x9001, &hf_h248_pkg_annexc_tmr, h248_param_ber_integer, NULL },
+	{ 0x9002, &hf_h248_pkg_annexc_tmrsr, h248_param_ber_integer, NULL },
+	{ 0x9003, &hf_h248_pkg_annexc_contcheck, h248_param_ber_integer, NULL },
+	{ 0x9004, &hf_h248_pkg_annexc_itc, h248_param_ber_integer, NULL },
+	{ 0x9005, &hf_h248_pkg_annexc_transmode, h248_param_ber_integer, NULL },
+	{ 0x9006, &hf_h248_pkg_annexc_transrate, h248_param_ber_integer, NULL },
+	{ 0x9007, &hf_h248_pkg_annexc_mult, h248_param_ber_integer, NULL },
 	{ 0x9008, &hf_h248_pkg_annexc_USI, dissect_h248_annexc_USI, NULL },
-	{ 0x9009, &hf_h248_pkg_annexc_syncasync, dissect_byte_param, NULL },
-	{ 0x900A, &hf_h248_pkg_annexc_negotiation, dissect_byte_param, NULL },
-	{ 0x900B, &hf_h248_pkg_annexc_userrate, dissect_byte_param, NULL },
-	{ 0x900C, &hf_h248_pkg_annexc_intrate, dissect_byte_param, NULL },
+	{ 0x9009, &hf_h248_pkg_annexc_syncasync, h248_param_ber_integer, NULL },
+	{ 0x900A, &hf_h248_pkg_annexc_negotiation, h248_param_ber_integer, NULL },
+	{ 0x900B, &hf_h248_pkg_annexc_userrate, h248_param_ber_integer, NULL },
+	{ 0x900C, &hf_h248_pkg_annexc_intrate, h248_param_ber_integer, NULL },
 	{ 0x900D, &hf_h248_pkg_annexc_nictx, h248_param_ber_boolean, NULL },
 	{ 0x900E, &hf_h248_pkg_annexc_nicrx, h248_param_ber_boolean, NULL },
 	{ 0x900F, &hf_h248_pkg_annexc_flowconttx, h248_param_ber_boolean, NULL },
@@ -905,22 +902,22 @@ static h248_pkg_param_t h248_annexc_package_properties[] = {
 	{ 0x9014, &hf_h248_pkg_annexc_llidnegot, h248_param_ber_boolean, NULL },
 	{ 0x9015, &hf_h248_pkg_annexc_assign, h248_param_ber_boolean, NULL },
 	{ 0x9016, &hf_h248_pkg_annexc_inbandneg, h248_param_ber_boolean, NULL },
-	{ 0x9017, &hf_h248_pkg_annexc_stopbits, dissect_byte_param, NULL },
-	{ 0x9018, &hf_h248_pkg_annexc_databits, dissect_byte_param, NULL },
-	{ 0x9019, &hf_h248_pkg_annexc_parity, dissect_byte_param, NULL },
-	{ 0x901a, &hf_h248_pkg_annexc_duplexmode, dissect_byte_param, NULL },
-	{ 0x901b, &hf_h248_pkg_annexc_modem, dissect_byte_param, NULL },
-	{ 0x901c, &hf_h248_pkg_annexc_layer2prot, dissect_byte_param, NULL },
-	{ 0x901d, &hf_h248_pkg_annexc_layer3prot, dissect_byte_param, NULL },
-	{ 0x901e, &hf_h248_pkg_annexc_addlayer3prot, dissect_byte_param, NULL },
-	{ 0x901f, &hf_h248_pkg_annexc_dialedn, h248_param_item, &thirty },
-	{ 0x9020, &hf_h248_pkg_annexc_dialingn, h248_param_item, &thirty },
+	{ 0x9017, &hf_h248_pkg_annexc_stopbits, h248_param_ber_integer, NULL },
+	{ 0x9018, &hf_h248_pkg_annexc_databits, h248_param_ber_integer, NULL },
+	{ 0x9019, &hf_h248_pkg_annexc_parity, h248_param_ber_integer, NULL },
+	{ 0x901a, &hf_h248_pkg_annexc_duplexmode, h248_param_ber_integer, NULL },
+	{ 0x901b, &hf_h248_pkg_annexc_modem, h248_param_ber_integer, NULL },
+	{ 0x901c, &hf_h248_pkg_annexc_layer2prot, h248_param_ber_integer, NULL },
+	{ 0x901d, &hf_h248_pkg_annexc_layer3prot, h248_param_ber_integer, NULL },
+	{ 0x901e, &hf_h248_pkg_annexc_addlayer3prot, h248_param_ber_integer, NULL },
+	{ 0x901f, &hf_h248_pkg_annexc_dialedn, h248_param_ber_octetstring, NULL },
+	{ 0x9020, &hf_h248_pkg_annexc_dialingn, h248_param_ber_octetstring, NULL },
 	{ 0x9021, &hf_h248_pkg_annexc_echoci, h248_param_ber_integer, NULL },
-	{ 0x9022, &hf_h248_pkg_annexc_nci, dissect_byte_param, NULL },
+	{ 0x9022, &hf_h248_pkg_annexc_nci, h248_param_ber_integer, NULL },
 	{ 0x9023, &hf_h248_pkg_annexc_USI, dissect_h248_annexc_USI, NULL },
 	
-	{ 0xA001, &hf_h248_pkg_annexc_fmsdu, h248_param_item, &four },
-	{ 0xA002, &hf_h248_pkg_annexc_bmsdu, h248_param_item, &four },
+	{ 0xA001, &hf_h248_pkg_annexc_fmsdu, h248_param_ber_octetstring, NULL },
+	{ 0xA002, &hf_h248_pkg_annexc_bmsdu, h248_param_ber_octetstring, NULL },
 	{ 0xA003, &hf_h248_pkg_annexc_sscs, NULL, NULL },
 
 	{ 0xB001, &hf_h248_pkg_annexc_sdp_v, h248_param_ber_octetstring, &h248_c_implicit },
@@ -952,8 +949,11 @@ static h248_pkg_param_t h248_annexc_package_properties[] = {
 static h248_package_t h248_annexc_package = {
 	0x0000,
 	&proto_h248_pkg_annexc,
-	&hf_h248_pkg_annexc_parameters,
 	&ett_annexc,
+	h248_annexc_package_properties_vals,
+	NULL,
+	NULL,
+	NULL,
 	h248_annexc_package_properties, /* properties */
 	NULL,							/* signals */
 	NULL,							/* events */
@@ -963,8 +963,6 @@ static h248_package_t h248_annexc_package = {
 
 void proto_register_h248_annex_c(void) {
 	static hf_register_info hf[] = {
-		{ &hf_h248_pkg_annexc_parameters,
-			{ "Parameter", "h248.pkg.annexc.parameter", FT_UINT16, BASE_HEX, VALS(h248_annexc_package_properties_vals), 0, "Annex-C Parameter ID", HFILL }},
 		{ &hf_h248_pkg_annexc_media,
 			{ "Media", "h248.pkg.annexc.media", FT_UINT32, BASE_HEX, VALS(h248_annexc_media_vals), 0, "Media Type", HFILL }},
 		{ &hf_h248_pkg_annexc_ACodec,
