@@ -4,7 +4,7 @@
  *
  * See RFC 3244 
  *
- * $Id: packet-kpasswd.c 19945 2006-11-21 08:44:21Z sahlberg $
+ * $Id: packet-kpasswd.c 21889 2007-05-22 16:17:54Z etxrab $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -31,6 +31,7 @@
 
 #include <epan/packet.h>
 #include <epan/emem.h>
+#include <epan/asn1.h>
 #include "packet-tcp.h"
 #include "packet-kerberos.h"
 #include "packet-ber.h"
@@ -82,14 +83,14 @@ dissect_kpasswd_ap_req_data(packet_info *pinfo _U_, tvbuff_t *tvb, proto_tree *p
 }
 
 
-static int dissect_kpasswd_newpassword(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb, int offset)
+static int dissect_kpasswd_newpassword(proto_tree *tree, tvbuff_t *tvb, int offset, asn1_ctx_t *actx _U_)
 {
-	offset=dissect_ber_octet_string_wcb(FALSE, pinfo, tree, tvb, offset, hf_kpasswd_newpassword, NULL);
+	offset=dissect_ber_octet_string_wcb(FALSE, actx, tree, tvb, offset, hf_kpasswd_newpassword, NULL);
 
 	return offset;
 }
 
-static ber_sequence_t ChangePasswdData_sequence[] = {
+static ber_old_sequence_t ChangePasswdData_sequence[] = {
 	{ BER_CLASS_CON, 0, 0,
 		dissect_kpasswd_newpassword },
 	{ BER_CLASS_CON, 1, BER_FLAGS_OPTIONAL, 
@@ -103,8 +104,10 @@ static int
 dissect_kpasswd_user_data_request(packet_info *pinfo, tvbuff_t *tvb, proto_tree *tree)
 {
     int offset=0;
+	asn1_ctx_t asn1_ctx;
+	asn1_ctx_init(&asn1_ctx, ASN1_ENC_BER, TRUE, pinfo);
 
-    offset=dissect_ber_sequence(FALSE, pinfo, tree, tvb, offset, ChangePasswdData_sequence, hf_kpasswd_ChangePasswdData, ett_ChangePasswdData);
+    offset=dissect_ber_old_sequence(FALSE, &asn1_ctx, tree, tvb, offset, ChangePasswdData_sequence, hf_kpasswd_ChangePasswdData, ett_ChangePasswdData);
 
     return offset;
 }
