@@ -3,7 +3,7 @@
  * By Mike Frisch <mfrisch@platform.com>
  * Joint and Several Copyright 2005, Mike Frisch and Platform Computing Inc.
  *
- * $Id: packet-pvfs2.c 19895 2006-11-14 20:32:29Z jmayer $
+ * $Id: packet-pvfs2.c 21142 2007-03-23 01:29:52Z ulfl $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -3070,7 +3070,7 @@ pvfs2_io_tracking_hash(gconstpointer k)
 {
 	const pvfs2_io_tracking_key_t *key = (const pvfs2_io_tracking_key_t *) k;
 
-	return (key->tag >> 32) ^ ((guint32) key->tag);
+	return (guint) ((key->tag >> 32) ^ ((guint32) key->tag));
 }
 
 static void
