@@ -2,7 +2,7 @@
  * Wireshark memory management and garbage collection functions
  * Ronnie Sahlberg 2005
  *
- * $Id: emem.c 20624 2007-01-30 17:14:37Z gerald $
+ * $Id: emem.c 21978 2007-05-29 06:56:07Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -40,16 +40,16 @@
 #include <unistd.h>
 #endif
 
-#ifdef _WIN32
-#include <windows.h>	/* VirtualAlloc, VirtualProtect */
-#include <process.h>    /* getpid */
-#endif
-
 #include <glib.h>
 #include <proto.h>
 #include "emem.h"
 #include <wiretap/file_util.h>
 
+#ifdef _WIN32
+#include <windows.h>	/* VirtualAlloc, VirtualProtect */
+#include <process.h>    /* getpid */
+#endif
+
 
 /*
  * Tools like Valgrind and ElectricFence don't work well with memchunks.
@@ -866,7 +866,7 @@ void print_tree(emem_tree_node_t *node){
 emem_tree_t *se_trees=NULL;
 
 emem_tree_t *
-se_tree_create(int type, char *name)
+se_tree_create(int type, const char *name)
 {
 	emem_tree_t *tree_list;
 
@@ -1321,7 +1321,7 @@ static void* lookup_or_insert32(emem_tree_t *se_tree, guint32 key, void*(*func)(
  * never existed including all metadata associated with the tree.
  */
 emem_tree_t *
-se_tree_create_non_persistent(int type, char *name)
+se_tree_create_non_persistent(int type, const char *name)
 {
 	emem_tree_t *tree_list;
 
@@ -1431,24 +1431,29 @@ void
 emem_tree_insert_string(emem_tree_t* se_tree, const gchar* k, void* v) {
 	guint32 len = strlen(k);
 	guint32 div = (len-1)/4;
+	guint32 *aligned;
 	guint32 residual = 0;
-	emem_tree_key_t key[] = {
-		{1,NULL},
-		{0,NULL},
-		{1,NULL},
-		{0,NULL}
-	};
+	emem_tree_key_t key[4];
 
+	aligned = malloc(div * sizeof (guint32));
+	if (aligned == NULL)
+		return;	/* XXX - fail somehow? */
+	memcpy(aligned, k, div * sizeof (guint32));
+
+	key[0].length = 1;
 	key[0].key = &len;
-	key[1].length = div;
-	key[1].key = (guint32*)(&k[0]);
-	key[2].key = &residual;
-
 	if (! div) {
-		key[1].length = key[2].length;
-		key[1].key = key[2].key;
+		key[1].length = 1;
+		key[1].key = &residual;
 		key[2].length = 0;
 		key[2].key = NULL;
+	} else {
+		key[1].length = div;
+		key[1].key = aligned;
+		key[2].length = 1;
+		key[2].key = &residual;
+		key[3].length = 0;
+		key[3].key = NULL;
 	}
 
 	div *= 4;
@@ -1466,30 +1471,37 @@ emem_tree_insert_string(emem_tree_t* se_tree, const gchar* k, void* v) {
 	}
 
 	emem_tree_insert32_array(se_tree,key,v);
+	free(aligned);
 }
 
 void *
 emem_tree_lookup_string(emem_tree_t* se_tree, const gchar* k) {
 	guint32 len = strlen(k);
 	guint32 div = (len-1)/4;
+	guint32 *aligned;
 	guint32 residual = 0;
-	emem_tree_key_t key[] = {
-		{1,NULL},
-		{0,NULL},
-		{1,NULL},
-		{0,NULL}
-	};
+	emem_tree_key_t key[4];
+	void *ret;
 
+	aligned = malloc(div * sizeof (guint32));
+	if (aligned == NULL)
+		return NULL;	/* XXX - better failure indication? */
+	memcpy(aligned, k, div * sizeof (guint32));
+
+	key[0].length = 1;
 	key[0].key = &len;
-	key[1].length = div;
-	key[1].key = (guint32*)(&k[0]);
-	key[2].key = &residual;
-
 	if (! div) {
-		key[1].length = key[2].length;
-		key[1].key = key[2].key;
+		key[1].length = 1;
+		key[1].key = &residual;
 		key[2].length = 0;
 		key[2].key = NULL;
+	} else {
+		key[1].length = div;
+		key[1].key = aligned;
+		key[2].length = 1;
+		key[2].key = &residual;
+		key[3].length = 0;
+		key[3].key = NULL;
 	}
 
 	div *= 4;
@@ -1506,7 +1518,9 @@ emem_tree_lookup_string(emem_tree_t* se_tree, const gchar* k) {
 			break;
 	}
 
-	return emem_tree_lookup32_array(se_tree, key);
+	ret = emem_tree_lookup32_array(se_tree, key);
+	free(aligned);
+	return ret;
 }
 
 
@@ -1522,7 +1536,7 @@ emem_tree_print_nodes(emem_tree_node_t* node, int level)
 		printf("    ");
 	}
 
-	printf("NODE:%08x parent:%08x left:0x%08x right:%08x key:%d data:0x%08x\n",(int)node,(int)node->parent,(int)node->left,(int)node->right,node->key32,(int)node->data);
+	printf("NODE:%p parent:%p left:0x%p right:%px key:%d data:%p\n",node,node->parent,node->left,node->right,node->key32,node->data);
 	if(node->left)
 		emem_tree_print_nodes(node->left, level+1);
 	if(node->right)
@@ -1534,7 +1548,7 @@ emem_print_tree(emem_tree_t* emem_tree)
 	if (!emem_tree)
 		return;
 
-	printf("EMEM tree type:%d name:%s tree:0x%08x\n",emem_tree->type,emem_tree->name,(int)emem_tree->tree);
+	printf("EMEM tree type:%d name:%s tree:%p\n",emem_tree->type,emem_tree->name,emem_tree->tree);
 	if(emem_tree->tree)
 		emem_tree_print_nodes(emem_tree->tree, 0);
 }
