@@ -1,7 +1,7 @@
 /* capture_file_dlg.c
  * Dialog boxes for handling capture files
  *
- * $Id: capture_file_dlg.c 19814 2006-11-05 22:46:44Z ulfl $
+ * $Id: capture_file_dlg.c 21990 2007-05-29 18:52:42Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -177,7 +177,7 @@ preview_set_filename(GtkWidget *prev, const gchar *cf_name)
         wtap_close(wth);
         return NULL;
     }
-    g_snprintf(string_buff, PREVIEW_STR_MAX, "%" PRId64 " bytes", filesize);
+    g_snprintf(string_buff, PREVIEW_STR_MAX, "%" G_GINT64_MODIFIER "d bytes", filesize);
     label = OBJECT_GET_DATA(prev, PREVIEW_SIZE_KEY);
     gtk_label_set_text(GTK_LABEL(label), string_buff);
 
@@ -215,7 +215,7 @@ preview_do(GtkWidget *prev, wtap *wth)
     time(&time_preview);
     while ( (wtap_read(wth, &err, &err_info, &data_offset)) ) {
         phdr = wtap_phdr(wth);
-        cur_time = nstime_to_sec( (const nstime_t *) &phdr->ts );
+        cur_time = wtap_nstime_to_sec(&phdr->ts);
         if(packets == 0) {
             start_time 	= cur_time;
             stop_time = cur_time;
@@ -1044,14 +1044,14 @@ file_merge_ok_cb(GtkWidget *w, gpointer fs) {
       rb = OBJECT_GET_DATA(w, E_MERGE_PREPEND_KEY);
       if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (rb))) {
           /* prepend file */
-          in_filenames[0] = cfile.filename;
-          in_filenames[1] = cf_name;
+          in_filenames[0] = cf_name;
+          in_filenames[1] = cfile.filename;
           merge_status = cf_merge_files(&tmpname, 2, in_filenames, filetype,
                                         TRUE);
       } else {
           /* append file */
-          in_filenames[0] = cf_name;
-          in_filenames[1] = cfile.filename;
+          in_filenames[0] = cfile.filename;
+          in_filenames[1] = cf_name;
           merge_status = cf_merge_files(&tmpname, 2, in_filenames, filetype,
                                         TRUE);
       }
@@ -1621,7 +1621,7 @@ color_global_cb(GtkWidget *widget _U_, gpointer data)
 
 /* Import color filters */
 void
-file_color_import_cmd_cb(GtkWidget *color_filters, gpointer filter_list)
+file_color_import_cmd_cb(GtkWidget *color_filters, gpointer filter_list _U_)
 {
 #if GTK_MAJOR_VERSION >= 2 && _WIN32
   win32_import_color_file(GDK_WINDOW_HWND(top_level->window), color_filters);
