@@ -4,7 +4,7 @@
  * conversations_table   2003 Ronnie Sahlberg
  * Helper routines common to all endpoint conversations tap.
  *
- * $Id: conversations_table.c 19568 2006-10-17 01:29:55Z ulfl $
+ * $Id: conversations_table.c 21990 2007-05-29 18:52:42Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -108,8 +108,8 @@ ct_port_to_str(int port_type, guint32 port)
    return a string for the filter name
 
    some addresses, like AT_ETHER may actually be any of multiple types
-   of protocols,   either ethernet, tokenring, fddi etc so we must be more
-   specific there  thats why we need specific_addr_type
+   of protocols,   either ethernet, tokenring, fddi, wlan etc so we must be
+   more specific there  thats why we need specific_addr_type
 */
 static const char *
 ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int name_type)
@@ -121,6 +121,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
 			switch(specific_addr_type){
 			case SAT_ETHER:
 				return "eth.src";
+			case SAT_WLAN:
+				return "wlan.sa";
 			case SAT_FDDI:
 				return "fddi.src";
 			case SAT_TOKENRING:
@@ -145,6 +147,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
                             ;
 			}
                         break;
+		case AT_USB:
+			return "usb.sa";
 		default:
 			;
 		}
@@ -155,6 +159,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
 			switch(specific_addr_type){
 			case SAT_ETHER:
 				return "eth.dst";
+			case SAT_WLAN:
+				return "wlan.da";
 			case SAT_FDDI:
 				return "fddi.dst";
 			case SAT_TOKENRING:
@@ -179,6 +185,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
                             ;
 			}
                         break;
+		case AT_USB:
+			return "usb.da";
 		default:
 			;
 		}
@@ -189,6 +197,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
 			switch(specific_addr_type){
 			case SAT_ETHER:
 				return "eth.addr";
+			case SAT_WLAN:
+				return "wlan.addr";
 			case SAT_FDDI:
 				return "fddi.addr";
 			case SAT_TOKENRING:
@@ -213,6 +223,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
                             ;
 			}
                         break;
+		case AT_USB:
+			return "usb.addr";
 		default:
 			;
 		}
@@ -1026,7 +1038,7 @@ draw_ct_table_address(conversations_table *ct, int conversation_idx)
     guint32 pt;
     int rownum;
 
-    rownum=gtk_clist_find_row_from_data(ct->table, (gpointer)conversation_idx);
+    rownum=gtk_clist_find_row_from_data(ct->table, (gpointer)(long)conversation_idx);
 
     if(!ct->resolve_names)
         entry=address_to_str(&ct->conversations[conversation_idx].src_address);
@@ -1103,23 +1115,23 @@ draw_ct_table_data(conversations_table *ct)
     for(i=0;i<ct->num_conversations;i++){
         char str[16];
 
-        j=gtk_clist_find_row_from_data(ct->table, (gpointer)i);
+        j=gtk_clist_find_row_from_data(ct->table, (gpointer)(unsigned long)i);
 
-        g_snprintf(str, 16, "%" PRIu64, ct->conversations[i].tx_frames+ct->conversations[i].rx_frames);
+        g_snprintf(str, 16, "%" G_GINT64_MODIFIER "u", ct->conversations[i].tx_frames+ct->conversations[i].rx_frames);
         gtk_clist_set_text(ct->table, j, 4, str);
-        g_snprintf(str, 16, "%" PRIu64, ct->conversations[i].tx_bytes+ct->conversations[i].rx_bytes);
+        g_snprintf(str, 16, "%" G_GINT64_MODIFIER "u", ct->conversations[i].tx_bytes+ct->conversations[i].rx_bytes);
         gtk_clist_set_text(ct->table, j, 5, str);
 
 
-        g_snprintf(str, 16, "%" PRIu64, ct->conversations[i].tx_frames);
+        g_snprintf(str, 16, "%" G_GINT64_MODIFIER "u", ct->conversations[i].tx_frames);
         gtk_clist_set_text(ct->table, j, 6, str);
-        g_snprintf(str, 16, "%" PRIu64, ct->conversations[i].tx_bytes);
+        g_snprintf(str, 16, "%" G_GINT64_MODIFIER "u", ct->conversations[i].tx_bytes);
         gtk_clist_set_text(ct->table, j, 7, str);
 
 
-        g_snprintf(str, 16, "%" PRIu64, ct->conversations[i].rx_frames);
+        g_snprintf(str, 16, "%" G_GINT64_MODIFIER "u", ct->conversations[i].rx_frames);
         gtk_clist_set_text(ct->table, j, 8, str);
-        g_snprintf(str, 16, "%" PRIu64, ct->conversations[i].rx_bytes);
+        g_snprintf(str, 16, "%" G_GINT64_MODIFIER "u", ct->conversations[i].rx_bytes);
         gtk_clist_set_text(ct->table, j, 9, str);
 
     }
@@ -1656,23 +1668,23 @@ add_conversation_table_data(conversations_table *ct, const address *src, const a
         entries[2] = "";
         entries[3] = "";
 
-        g_snprintf(frames, 16, "%" PRIu64, conversation->tx_frames+conversation->rx_frames);
+        g_snprintf(frames, 16, "%" G_GINT64_MODIFIER "u", conversation->tx_frames+conversation->rx_frames);
         entries[4]=frames;
-        g_snprintf(bytes, 16, "%" PRIu64, conversation->tx_bytes+conversation->rx_bytes);
+        g_snprintf(bytes, 16, "%" G_GINT64_MODIFIER "u", conversation->tx_bytes+conversation->rx_bytes);
         entries[5]=bytes;
 
-        g_snprintf(txframes, 16, "%" PRIu64, conversation->tx_frames);
+        g_snprintf(txframes, 16, "%" G_GINT64_MODIFIER "u", conversation->tx_frames);
         entries[6]=txframes;
-        g_snprintf(txbytes, 16, "%" PRIu64, conversation->tx_bytes);
+        g_snprintf(txbytes, 16, "%" G_GINT64_MODIFIER "u", conversation->tx_bytes);
         entries[7]=txbytes;
 
-        g_snprintf(rxframes, 16, "%" PRIu64, conversation->rx_frames);
+        g_snprintf(rxframes, 16, "%" G_GINT64_MODIFIER "u", conversation->rx_frames);
         entries[8]=rxframes;
-        g_snprintf(rxbytes, 16, "%" PRIu64, conversation->rx_bytes);
+        g_snprintf(rxbytes, 16, "%" G_GINT64_MODIFIER "u", conversation->rx_bytes);
         entries[9]=rxbytes;
 
         gtk_clist_insert(ct->table, conversation_idx, entries);
-        gtk_clist_set_row_data(ct->table, conversation_idx, (gpointer) conversation_idx);
+        gtk_clist_set_row_data(ct->table, conversation_idx, (gpointer)(long) conversation_idx);
 
         draw_ct_table_address(ct, conversation_idx);
     }
