@@ -1,6 +1,6 @@
 /* follow_dlg.c
  *
- * $Id: follow_dlg.c 20017 2006-11-29 20:45:02Z sfisher $
+ * $Id: follow_dlg.c 21870 2007-05-22 00:00:09Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -101,7 +101,7 @@ typedef struct {
 	GtkWidget	*carray_bt;
 	GtkWidget	*raw_bt;
 	GtkWidget	*follow_save_as_w;
-#if GTK_MAJOR_VERSION >= 2
+#if GTK_CHECK_VERSION(2,4,0)
 	GtkWidget	*find_dlg_w;
 #endif
 	gboolean        is_ipv6;
@@ -114,7 +114,7 @@ static void follow_destroy_cb(GtkWidget * win, gpointer data);
 static void follow_charset_toggle_cb(GtkWidget * w, gpointer parent_w);
 static void follow_load_text(follow_info_t *follow_info);
 static void follow_filter_out_stream(GtkWidget * w, gpointer parent_w);
-#if GTK_MAJOR_VERSION >= 2
+#if GTK_CHECK_VERSION(2,4,0)
 static void follow_find_cb(GtkWidget * w, gpointer data);
 static void follow_find_button_cb(GtkWidget * w _U_, gpointer parent_w);
 static void follow_find_destroy_cb(GtkWidget * win _U_, gpointer data);
@@ -185,6 +185,7 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 	follow_tcp_stats_t stats;
 	follow_info_t	*follow_info;
 	tcp_stream_chunk sc;
+	size_t              nchars;
 
 	/* we got tcp so we can follow */
 	if (cfile.edt->pi.ipproto != IP_PROTO_TCP) {
@@ -255,7 +256,6 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
             "!(%s)", follow_filter);
 	}
 
-
 	gtk_entry_set_text(GTK_ENTRY(filter_te), follow_filter);
 
 	/* Run the display filter so it goes in effect - even if it's the
@@ -265,6 +265,47 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 	/* Free the filter string, as we're done with it. */
 	g_free(follow_filter);
 
+	/* Check whether we got any data written to the file. */
+	if (empty_tcp_stream) {
+	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+			  "The packets in the capture file for that stream have no data.");
+	    eth_close(tmp_fd);
+	    unlink(follow_info->data_out_filename);
+	    g_free(follow_info);
+	    return;
+	}
+
+	/* Go back to the top of the file and read the first tcp_stream_chunk
+	 * to ensure that the IP addresses and port numbers in the drop-down
+	 * list are tied to the correct lines displayed by follow_read_stream()
+	 * later on (which also reads from this file).  Close the file when
+	 * we're done.
+	 *
+	 * We read the data now, before we pop up a window, in case the
+	 * read fails.  We use the data later.
+	 */
+
+	rewind(data_out_file);
+	nchars=fread(&sc, 1, sizeof(sc), data_out_file);
+	if (nchars != sizeof(sc)) {
+	    if (ferror(data_out_file)) {
+		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+			      "Could not read from temporary file %s: %s",
+			      follow_info->data_out_filename, strerror(errno));
+	    } else {
+		simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+			      "Short read from temporary file %s: expected %lu, got %lu",
+			      follow_info->data_out_filename,
+			      (unsigned long)sizeof(sc),
+			      (unsigned long)nchars);
+	    }
+	    eth_close(tmp_fd);
+	    unlink(follow_info->data_out_filename);
+	    g_free(follow_info);
+	    return;
+	}
+	fclose(data_out_file);
+
 	/* The data_out_filename file now has all the text that was in the session */
 	streamwindow = dlg_window_new("Follow TCP Stream");
 
@@ -319,7 +360,7 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 	hbox = gtk_hbox_new(FALSE, 1);
 	gtk_box_pack_start(GTK_BOX(stream_vb), hbox, FALSE, FALSE, 0);
 
-#if GTK_MAJOR_VERSION >= 2
+#if GTK_CHECK_VERSION(2,4,0)
 	/* Create Find Button */
 	button = BUTTON_NEW_FROM_STOCK(GTK_STOCK_FIND);
 	SIGNAL_CONNECT(button, "clicked", follow_find_cb, follow_info);
@@ -375,17 +416,6 @@ follow_stream_cb(GtkWidget * w, gpointer data _U_)
 	gtk_widget_show(stream_mi);
 	follow_info->show_stream = BOTH_HOSTS;
 
-	/* Go back to the top of the file and read the first tcp_stream_chunk
-	 * to ensure that the IP addresses and port numbers in the drop-down
-	 * list are tied to the correct lines displayed by follow_read_stream()
-	 * later on (which also reads from this file).  Close the file when
-	 * we're done.
-	 */
-
-	rewind(data_out_file);
-	fread(&sc, 1, sizeof(sc), data_out_file);
-	fclose(data_out_file);
-
 	/* Host 0 --> Host 1 */
 	if(sc.src_port == strtol(port0, NULL, 10)) {
 		g_snprintf(string, sizeof(string), "%s:%s --> %s:%s (%u bytes)",
@@ -660,7 +690,17 @@ follow_read_stream(follow_info_t *follow_info,
 	return FRS_OPEN_ERROR;
     }
 
-    while (fread(&sc, 1, sizeof(sc), data_out_file)) {
+    while ((nchars=fread(&sc, 1, sizeof(sc), data_out_file))) {
+    	if (nchars != sizeof(sc)) {
+	    simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+			  "Short read from temporary file %s: expected %lu, got %lu",
+			  follow_info->data_out_filename,
+			  (unsigned long)sizeof(sc),
+			  (unsigned long)nchars);
+	    fclose(data_out_file);
+	    data_out_file = NULL;
+	    return FRS_READ_ERROR;
+	}
 	if (client_port == 0) {
 	    memcpy(client_addr, sc.src_addr, iplen);
 	    client_port = sc.src_port;
@@ -687,6 +727,7 @@ follow_read_stream(follow_info_t *follow_info,
 	    nchars = fread(buffer, 1, bcount, data_out_file);
 	    if (nchars == 0)
 		break;
+	    /* XXX - if we don't get "bcount" bytes, is that an error? */
 	    sc.dlen -= nchars;
 
 	    if (!skip) {
@@ -817,8 +858,8 @@ follow_read_stream(follow_info_t *follow_info,
 	return FRS_READ_ERROR;
     }
 
-	fclose(data_out_file);
-	data_out_file = NULL;
+    fclose(data_out_file);
+    data_out_file = NULL;
     return FRS_OK;
 
 print_error:
@@ -893,7 +934,7 @@ follow_filter_out_stream(GtkWidget * w _U_, gpointer data)
     return;
 }
 
-#if GTK_MAJOR_VERSION >= 2
+#if GTK_CHECK_VERSION(2,4,0)
 static void
 follow_find_cb(GtkWidget * w _U_, gpointer data)
 {
@@ -1013,7 +1054,7 @@ follow_find_destroy_cb(GtkWidget * win _U_, gpointer data)
 	/* Note that we no longer have a dialog box. */
 	follow_info->find_dlg_w = NULL;
 }
-#endif /* GTK_MAJOR_VERSION >= 2 */
+#endif /* GTK_CHECK_VERSION(2,4,0) */
 
 static void
 follow_print_stream(GtkWidget * w _U_, gpointer data)
@@ -1153,7 +1194,25 @@ follow_add_to_gtk_text(char *buffer, size_t nchars, gboolean is_server,
      * to be able to see the data we *should* see
      * in the GtkText widget.
      */
-    size_t i;
+    size_t i,j;
+    char *str;
+
+    /* XXX - workaround for bug 598 
+     * convert CRLF to LF because the GTK copy function seems
+     * to convert CR to LF, resulting in double LF's
+     */
+    str = g_malloc(nchars + 1);
+    for (i = 0,j = 0; i < nchars; i++) {
+        if (i>0 && buffer[i-1] == '\r' && buffer[i] == '\n')
+	    str[j-1] = buffer[i];
+	else
+	    str[j++] = buffer[i];
+    }
+    nchars = nchars + j - i;
+    memcpy(buffer, str, nchars);
+    buffer[nchars] = 0;
+    g_free(str);
+    /* end of workaround for bug 598 */
 
     for (i = 0; i < nchars; i++) {
         if (buffer[i] == '\n' || buffer[i] == '\r')
