@@ -6,7 +6,7 @@
  *
  * Modified from summary_dlg.c
  *
- * $Id: gsm_map_summary.c 19814 2006-11-05 22:46:44Z ulfl $
+ * $Id: gsm_map_summary.c 21978 2007-05-29 06:56:07Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -48,6 +48,7 @@
 #include "compat_macros.h"
 #include <epan/tap.h>
 
+#include <epan/asn1.h>
 #include <epan/dissectors/packet-gsm_map.h>
 #include "gsm_map_stat.h"
 
@@ -110,7 +111,7 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   add_string_to_box(string_buff, file_box);
 
   /* length */
-  g_snprintf(string_buff, SUM_STR_MAX, "Length: %lld", summary.file_length);
+  g_snprintf(string_buff, SUM_STR_MAX, "Length: %" G_GINT64_MODIFIER "d", summary.file_length);
   add_string_to_box(string_buff, file_box);
 
   /* format */
