@@ -1,7 +1,7 @@
 /* layout_prefs.c
  * Dialog box for layout preferences
  *
- * $Id: layout_prefs.c 18205 2006-05-22 07:29:40Z guy $
+ * $Id: layout_prefs.c 21876 2007-05-22 02:21:01Z guy $
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
@@ -557,7 +557,7 @@ layout_prefs_fetch(GtkWidget *w)
 void
 layout_prefs_apply(GtkWidget *w _U_)
 {
-    set_main_window_name("The Wireshark Network Analyzer");
+    update_main_window_name();
     main_widgets_rearrange();
 }
 
