@@ -1,7 +1,7 @@
 /* voip_calls.c
  * VoIP calls summary addition for Wireshark
  *
- * $Id: voip_calls.c 19667 2006-10-24 05:17:49Z jake $
+ * $Id: voip_calls.c 22017 2007-05-31 18:41:25Z sfisher $
  *
  * Copyright 2004, Ericsson, Spain
  * By Francisco Alcoba <francisco.alcoba@ericsson.com>
@@ -59,8 +59,12 @@
 #include <epan/dissectors/packet-rtp.h>
 #include <epan/dissectors/packet-rtp-events.h>
 #include <epan/dissectors/packet-t38.h>
+#include <epan/dissectors/packet-t30.h>
+#include <epan/dissectors/packet-h248.h>
+#include <epan/dissectors/packet-sccp.h>
 #include <epan/conversation.h>
 #include <epan/rtp_pt.h>
+#include <epan/ws_strsplit.h>
 
 #include "alert_box.h"
 #include "simple_dialog.h"
@@ -71,7 +75,8 @@
 #endif /* GTK_MAJOR_VERSION >= 2 */
 #endif /* HAVE_LIBPORTAUDIO */
 
-const char *voip_call_state_name[7]={
+const char *voip_call_state_name[8]={
+	"",
 	"CALL SETUP",
 	"RINGING",
 	"IN CALL",
@@ -82,15 +87,19 @@ const char *voip_call_state_name[7]={
 	};
 
 /* defines whether we can consider the call active */
-const char *voip_protocol_name[7]={
+const char *voip_protocol_name[]={
 	"SIP",
 	"ISUP",
 	"H.323",
 	"MGCP",
 	"AC_ISDN",
 	"AC_CAS",
-	"T.38"
-	};
+	"T.38",
+	"H.248",
+	"SCCP",
+	"BSSMAP",
+	"RANAP"
+};
 
 typedef struct {
 	gchar *frame_label;
@@ -110,7 +119,7 @@ static h245_labels_t h245_labels;
 /****************************************************************************/
 /* the one and only global voip_calls_tapinfo_t structure */
 static voip_calls_tapinfo_t the_tapinfo_struct =
-	{0, NULL, 0, NULL, 0, 0, 0, 0, NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+	{0, NULL, 0, NULL, 0, 0, 0, 0, NULL, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
 
 /* the one and only global voip_rtp_tapinfo_t structure */
 static voip_rtp_tapinfo_t the_tapinfo_rtp_struct =
@@ -121,14 +130,8 @@ static voip_rtp_tapinfo_t the_tapinfo_rtp_struct =
 void voip_calls_reset(voip_calls_tapinfo_t *tapinfo)
 {
 	voip_calls_info_t *strinfo;
-	sip_calls_info_t *tmp_sipinfo;
-	h323_calls_info_t *tmp_h323info;
-	h245_address_t *h245_add;
-
 	graph_analysis_item_t *graph_item;
-
 	GList* list;
-	GList* list2;
 
 #ifdef HAVE_LIBPORTAUDIO
 #if GTK_MAJOR_VERSION >= 2
@@ -145,26 +148,9 @@ void voip_calls_reset(voip_calls_tapinfo_t *tapinfo)
 		g_free(strinfo->from_identity);
 		g_free(strinfo->to_identity);
 		g_free((void *)(strinfo->initial_speaker.data));
-		if (strinfo->protocol == VOIP_SIP){
-			tmp_sipinfo = strinfo->prot_info;
-			g_free(tmp_sipinfo->call_identifier);
-		}
-		if (strinfo->protocol == VOIP_H323){
-			tmp_h323info = strinfo->prot_info;
-			g_free(tmp_h323info->guid);
-			/* free the H245 list address */
-			list2 = g_list_first(tmp_h323info->h245_list);
-			while (list2)
-			{
-				h245_add=list2->data;
-				g_free((void *)h245_add->h245_address.data);
-				g_free(list2->data);
-				list2 = g_list_next(list2);
-			}
-			g_list_free(tmp_h323info->h245_list);
-			tmp_h323info->h245_list = NULL;
-		}
-		g_free(strinfo->prot_info);
+		
+		if (strinfo->free_prot_info && strinfo->prot_info)
+			strinfo->free_prot_info(strinfo->prot_info);
 
 		g_free(list->data);
 		list = g_list_next(list);
@@ -545,7 +531,7 @@ RTP_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, void con
 		if (!strinfo->pt_str) strinfo->pt_str = g_strdup(val_to_str(strinfo->pt, rtp_payload_type_short_vals, "%u"));
 		strinfo->npackets = 0;
 		strinfo->first_frame_num = pinfo->fd->num;
-		strinfo->start_rel_sec = pinfo->fd->rel_ts.secs;
+		strinfo->start_rel_sec = (guint32) (pinfo->fd->rel_ts.secs);
 		strinfo->start_rel_usec = pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->setup_frame_number = pi->info_setup_frame_num;
 		strinfo->rtp_event = -1;
@@ -555,7 +541,7 @@ RTP_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, void con
 	if (strinfo!=NULL){
 		/* Add the info to the existing RTP stream */
 		strinfo->npackets++;
-		strinfo->stop_rel_sec = pinfo->fd->rel_ts.secs;
+		strinfo->stop_rel_sec = (guint32) (pinfo->fd->rel_ts.secs);
 		strinfo->stop_rel_usec = pinfo->fd->rel_ts.nsecs/1000;
 
 		/* process RTP Event */
@@ -609,7 +595,7 @@ static void RTP_packet_draw(void *prs _U_)
 					if (rtp_listinfo->first_frame_num == gai->frame_num){
 						duration = (rtp_listinfo->stop_rel_sec*1000000 + rtp_listinfo->stop_rel_usec) - (rtp_listinfo->start_rel_sec*1000000 + rtp_listinfo->start_rel_usec);
 						g_free(gai->comment);
-						gai->comment = g_strdup_printf("RTP Num packets:%u  Duration:%u.%03us ssrc:%u", rtp_listinfo->npackets, duration/1000000,(duration%1000000)/1000, rtp_listinfo->ssrc);
+						gai->comment = g_strdup_printf("RTP Num packets:%u  Duration:%u.%03us SSRC:0x%X", rtp_listinfo->npackets, duration/1000000,(duration%1000000)/1000, rtp_listinfo->ssrc);
 						break;
 					}
 
@@ -629,7 +615,7 @@ static void RTP_packet_draw(void *prs _U_)
 						duration = (rtp_listinfo->stop_rel_sec*1000000 + rtp_listinfo->stop_rel_usec) - (rtp_listinfo->start_rel_sec*1000000 + rtp_listinfo->start_rel_usec);
 						new_gai->frame_label = g_strdup_printf("RTP (%s) %s", rtp_listinfo->pt_str, (rtp_listinfo->rtp_event == -1)?"":val_to_str(rtp_listinfo->rtp_event, rtp_event_type_values, "Unknown RTP Event")); 
 						g_free(rtp_listinfo->pt_str);
-						new_gai->comment = g_strdup_printf("RTP Num packets:%u  Duration:%u.%03us ssrc:%u", rtp_listinfo->npackets, duration/1000000,(duration%1000000)/1000, rtp_listinfo->ssrc);
+						new_gai->comment = g_strdup_printf("RTP Num packets:%u  Duration:%u.%03us SSRC:0x%X", rtp_listinfo->npackets, duration/1000000,(duration%1000000)/1000, rtp_listinfo->ssrc);
 						new_gai->conv_num = conv_num;
 						new_gai->display=FALSE;
 						new_gai->line_style = 2;  /* the arrow line will be 2 pixels width */
@@ -753,10 +739,11 @@ T38_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const vo
 			COPY_ADDRESS(&(strinfo->initial_speaker),&(pinfo->src));
 			strinfo->first_frame_num=pinfo->fd->num;
 			strinfo->selected=FALSE;
-			strinfo->start_sec=pinfo->fd->rel_ts.secs;
+			strinfo->start_sec=(gint32) (pinfo->fd->rel_ts.secs);
 			strinfo->start_usec=pinfo->fd->rel_ts.nsecs/1000;
 			strinfo->protocol=MEDIA_T38;
 			strinfo->prot_info=NULL;
+			strinfo->free_prot_info = NULL;
 			strinfo->npackets = 0;
 			strinfo->call_num = tapinfo->ncalls++;
 			tapinfo->strinfo_list = g_list_append(tapinfo->strinfo_list, strinfo);
@@ -775,8 +762,8 @@ T38_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const vo
 
 	/* add the item to the graph list */
 	if (pi->type_msg == 0) { /* 0=t30-indicator */
-		frame_label = g_strdup(val_to_str(pi->t30ind_value, t30_indicator_vals, "Ukn (0x%02X)") );
-		comment = g_strdup_printf("t38:t30 Ind:%s",val_to_str(pi->t30ind_value, t30_indicator_vals, "Ukn (0x%02X)") );
+		frame_label = g_strdup(val_to_str(pi->t30ind_value, t38_T30_indicator_vals, "Ukn (0x%02X)") );
+		comment = g_strdup_printf("t38:t30 Ind:%s",val_to_str(pi->t30ind_value, t38_T30_indicator_vals, "Ukn (0x%02X)") );
 		line_style = 1;
 	} else if (pi->type_msg == 1) {	/* 1=data */
 		switch(pi->Data_Field_field_type_value){
@@ -785,17 +772,17 @@ T38_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const vo
 			case 2: /* hdlc-fcs-OK */
 			case 4: /* hdlc-fcs-OK-sig-end */
 					frame_label = g_strdup_printf("%s %s", val_to_str(pi->t30_Facsimile_Control & 0x7F, t30_facsimile_control_field_vals_short, "Ukn (0x%02X)"), pi->desc);
-					comment = g_strdup_printf("t38:%s:HDLC:%s",val_to_str(pi->data_value, t30_data_vals, "Ukn (0x%02X)"), val_to_str(pi->t30_Facsimile_Control & 0x7F, t30_facsimile_control_field_vals, "Ukn (0x%02X)"));
+					comment = g_strdup_printf("t38:%s:HDLC:%s",val_to_str(pi->data_value, t38_T30_data_vals, "Ukn (0x%02X)"), val_to_str(pi->t30_Facsimile_Control & 0x7F, t30_facsimile_control_field_vals, "Ukn (0x%02X)"));
 				break;
 			case 3: /* hdlc-fcs-BAD */
 			case 5: /* hdlc-fcs-BAD-sig-end */
 				frame_label = g_strdup(pi->Data_Field_field_type_value == 3 ? "fcs-BAD" : "fcs-BAD-sig-end");
-				comment = g_strdup_printf("WARNING: received t38:%s:HDLC:%s", val_to_str(pi->data_value, t30_data_vals, "Ukn (0x%02X)"), pi->Data_Field_field_type_value == 3 ? "fcs-BAD" : "fcs-BAD-sig-end");
+				comment = g_strdup_printf("WARNING: received t38:%s:HDLC:%s", val_to_str(pi->data_value, t38_T30_data_vals, "Ukn (0x%02X)"), pi->Data_Field_field_type_value == 3 ? "fcs-BAD" : "fcs-BAD-sig-end");
 				break;
 			case 7: /* t4-non-ecm-sig-end */
 				duration = nstime_to_sec(&pinfo->fd->rel_ts) - pi->time_first_t4_data;
-				frame_label = g_strdup_printf("t4-non-ecm-data:%s",val_to_str(pi->data_value, t30_data_vals, "Ukn (0x%02X)") );
-				comment = g_strdup_printf("t38:t4-non-ecm-data:%s Duration: %.2fs %s",val_to_str(pi->data_value, t30_data_vals, "Ukn (0x%02X)"), duration, pi->desc_comment );
+				frame_label = g_strdup_printf("t4-non-ecm-data:%s",val_to_str(pi->data_value, t38_T30_data_vals, "Ukn (0x%02X)") );
+				comment = g_strdup_printf("t38:t4-non-ecm-data:%s Duration: %.2fs %s",val_to_str(pi->data_value, t38_T30_data_vals, "Ukn (0x%02X)"), duration, pi->desc_comment );
 				insert_to_graph(tapinfo, pinfo, frame_label, comment, (guint16)conv_num, &(pinfo->src), &(pinfo->dst), line_style, pi->time_first_t4_data, pi->frame_num_first_t4_data);
 				break;
 		}
@@ -858,6 +845,14 @@ static guint32 sdp_frame_num = 0;
 /* ***************************TAP for SIP **********************************/
 /****************************************************************************/
 
+
+static void free_sip_info(gpointer p) {
+	sip_calls_info_t *si = p;
+	
+	if (si->call_identifier) g_free(si->call_identifier);
+	g_free(si);
+}
+
 /****************************************************************************/
 /* whenever a SIP packet is seen by the tap listener */
 static int 
@@ -908,14 +903,15 @@ SIPcalls_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, con
 			COPY_ADDRESS(&(strinfo->initial_speaker),&(pinfo->src));
 			strinfo->first_frame_num=pinfo->fd->num;
 			strinfo->selected=FALSE;
-			strinfo->start_sec=pinfo->fd->rel_ts.secs;
+			strinfo->start_sec=(gint32) (pinfo->fd->rel_ts.secs);
 			strinfo->start_usec=pinfo->fd->rel_ts.nsecs/1000;
 			strinfo->protocol=VOIP_SIP;
 			strinfo->prot_info=g_malloc(sizeof(sip_calls_info_t));
-			tmp_sipinfo=strinfo->prot_info;
-			tmp_sipinfo->call_identifier=strdup(pi->tap_call_id);
-			tmp_sipinfo->sip_state=SIP_INVITE_SENT;
-			tmp_sipinfo->invite_cseq=pi->tap_cseq_number;
+			strinfo->free_prot_info = free_sip_info;
+			tmp_sipinfo = strinfo->prot_info;
+			tmp_sipinfo->call_identifier = g_strdup(pi->tap_call_id);
+			tmp_sipinfo->sip_state = SIP_INVITE_SENT;
+			tmp_sipinfo->invite_cseq = pi->tap_cseq_number;
 			strinfo->npackets = 0;
 			strinfo->call_num = tapinfo->ncalls++;
 			tapinfo->strinfo_list = g_list_append(tapinfo->strinfo_list, strinfo);
@@ -973,7 +969,7 @@ SIPcalls_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, con
 			}
 		}
 
-		strinfo->stop_sec=pinfo->fd->rel_ts.secs;
+		strinfo->stop_sec=(gint32) (pinfo->fd->rel_ts.secs);
 		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->last_frame_num=pinfo->fd->num;
 		++(strinfo->npackets);
@@ -1075,11 +1071,10 @@ isup_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 
 	/*voip_calls_tapinfo_t *tapinfo = &the_tapinfo_struct; unused */
 	const isup_tap_rec_t *pi = isup_info;
-
-
+	
 	/* check if the lower layer is MTP matching the frame number */
 	if (mtp3_frame_num != pinfo->fd->num) return 0;
-	
+		
 	/* check whether we already have a call with these parameters in the list */
 	list = g_list_first(tapinfo->strinfo_list);
 	while (list)
@@ -1091,26 +1086,24 @@ isup_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 			if ((tmp_isupinfo->cic == pinfo->circuit_id)&&(tmp_isupinfo->ni == mtp3_ni)) {
 				if ((tmp_isupinfo->opc == mtp3_opc)&&(tmp_isupinfo->dpc == mtp3_dpc)){
 					 forward = TRUE;
-				 }
-				 else if ((tmp_isupinfo->dpc == mtp3_opc)&&(tmp_isupinfo->opc == mtp3_dpc)){
+				} else if ((tmp_isupinfo->dpc == mtp3_opc)&&(tmp_isupinfo->opc == mtp3_dpc)){
 					 forward = FALSE;
-				 }
-				 else{
-					 right_pair = FALSE;
-				 }
+				} else{
+					right_pair = FALSE;
+				}
+				
 				 if (right_pair){
 					/* if there is an IAM for a call that is not in setup state, that means the previous call in the same
 					   cic is no longer active */
 					if (tmp_listinfo->call_state == VOIP_CALL_SETUP){
 						found = TRUE;
-					}
-					else if (pi->message_type != 1){
+					} else if (pi->message_type != 1){
 						found = TRUE;
-					}
-					else{
+					} else{
 						tmp_listinfo->call_active_state=VOIP_INACTIVE;
 					}
 				}
+				
 				if (found){
 					strinfo = (voip_calls_info_t*)(list->data);
 					break;
@@ -1125,14 +1118,13 @@ isup_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 
 
 	if ((strinfo==NULL) &&(pi->message_type==1)){
-
 		strinfo = g_malloc(sizeof(voip_calls_info_t));
 		strinfo->call_active_state = VOIP_ACTIVE;
 		strinfo->call_state = VOIP_UNKNOWN;
 		COPY_ADDRESS(&(strinfo->initial_speaker),&(pinfo->src));
 		strinfo->selected=FALSE;
 		strinfo->first_frame_num=pinfo->fd->num;
-		strinfo->start_sec=pinfo->fd->rel_ts.secs;
+		strinfo->start_sec=(gint32) (pinfo->fd->rel_ts.secs);
 		strinfo->start_usec=pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->protocol=VOIP_ISUP;
 		if (pi->calling_number!=NULL){
@@ -1142,6 +1134,7 @@ isup_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 			strinfo->to_identity=g_strdup(pi->called_number);
 		}
 		strinfo->prot_info=g_malloc(sizeof(isup_calls_info_t));
+		strinfo->free_prot_info = g_free;
 		tmp_isupinfo=strinfo->prot_info;
 		tmp_isupinfo->opc = mtp3_opc;
 		tmp_isupinfo->dpc = mtp3_dpc;
@@ -1152,8 +1145,9 @@ isup_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 		tapinfo->strinfo_list = g_list_append(tapinfo->strinfo_list, strinfo);
 	}
 
+	
 	if (strinfo!=NULL){
-		strinfo->stop_sec=pinfo->fd->rel_ts.secs;
+		strinfo->stop_sec=(gint32) (pinfo->fd->rel_ts.secs);
 		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->last_frame_num=pinfo->fd->num;
 		++(strinfo->npackets);
@@ -1171,26 +1165,24 @@ isup_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 		}
 
 		if (strinfo->npackets == 1){ /* this is the first packet, that must be an IAM */
+
 			if ((pi->calling_number!=NULL)&&(pi->called_number !=NULL)){
 				comment = g_strdup_printf("Call from %s to %s",
 				 pi->calling_number, pi->called_number);
 			 }
-		}
-		else if (strinfo->npackets == 2){ /* in the second packet we show the SPs */
+		} else if (strinfo->npackets == 2){ /* in the second packet we show the SPs */
 			if (forward){
 				comment = g_strdup_printf("%i-%i -> %i-%i. Cic:%i",
 				 mtp3_ni, mtp3_opc,
 				 mtp3_ni, mtp3_dpc, pinfo->circuit_id);
-
-			}
-			else{
+			} else {
 				comment = g_strdup_printf("%i-%i -> %i-%i. Cic:%i",
 				 mtp3_ni, mtp3_dpc,
 				 mtp3_ni, mtp3_opc, pinfo->circuit_id);
-
 			}
 		}
 
+
 		switch(pi->message_type){
 			case 1: /* IAM */
 				strinfo->call_state=VOIP_CALL_SETUP;
@@ -1304,6 +1296,7 @@ mtp3_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 /****************************************************************************/
 
 static gboolean have_mtp3_tap_listener=FALSE;
+static gboolean have_m3ua_tap_listener=FALSE;
 
 void
 mtp3_calls_init_tap(void)
@@ -1328,6 +1321,25 @@ mtp3_calls_init_tap(void)
 		}
 		have_mtp3_tap_listener=TRUE;
 	}
+	
+	if(have_m3ua_tap_listener==FALSE)
+	{
+		error_string = register_tap_listener("m3ua", &(the_tapinfo_struct.mtp3_dummy),
+											 NULL,
+											 voip_calls_dlg_reset, 
+											 mtp3_calls_packet, 
+											 voip_calls_dlg_draw
+											 );
+		
+		if (error_string != NULL) {
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+						  error_string->str);
+			g_string_free(error_string, TRUE);
+			exit(1);
+		}
+		have_m3ua_tap_listener=TRUE;
+	}
+	
 }
 
 /****************************************************************************/
@@ -1337,9 +1349,11 @@ remove_tap_listener_mtp3_calls(void)
 {
 	protect_thread_critical_region();
 	remove_tap_listener(&(the_tapinfo_struct.mtp3_dummy));
+	remove_tap_listener(&(the_tapinfo_struct.m3ua_dummy));
 	unprotect_thread_critical_region();
 
 	have_mtp3_tap_listener=FALSE;
+	have_m3ua_tap_listener=FALSE;
 }
 
 /****************************************************************************/
@@ -1559,10 +1573,11 @@ q931_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 			COPY_ADDRESS(&(strinfo->initial_speaker),actrace_direction?&pstn_add:&(pinfo->src));
 			strinfo->first_frame_num=pinfo->fd->num;
 			strinfo->selected=FALSE;
-			strinfo->start_sec=pinfo->fd->rel_ts.secs;
+			strinfo->start_sec=(gint32) (pinfo->fd->rel_ts.secs);
 			strinfo->start_usec=pinfo->fd->rel_ts.nsecs/1000;
 			strinfo->protocol=VOIP_AC_ISDN;
 			strinfo->prot_info=g_malloc(sizeof(actrace_isdn_calls_info_t));
+			strinfo->free_prot_info = g_free;
 			tmp_actrace_isdn_info=strinfo->prot_info;
 			tmp_actrace_isdn_info->crv=q931_crv;
 			tmp_actrace_isdn_info->trunk=actrace_trunk;
@@ -1571,7 +1586,7 @@ q931_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 			tapinfo->strinfo_list = g_list_append(tapinfo->strinfo_list, strinfo);
 		}
 
-		strinfo->stop_sec=pinfo->fd->rel_ts.secs;
+		strinfo->stop_sec=(gint32) (pinfo->fd->rel_ts.secs);
 		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->last_frame_num=pinfo->fd->num;
 		++(strinfo->npackets);
@@ -1675,6 +1690,28 @@ static void add_h245_Address(h323_calls_info_t *h323info,  h245_address_t *h245_
 	h323info->h245_list = g_list_append(h323info->h245_list, h245_address);				
 }
 
+
+static void free_h225_info(gpointer p) {
+	h323_calls_info_t *tmp_h323info = p;
+
+	if (tmp_h323info->guid) g_free(tmp_h323info->guid);
+
+	if (tmp_h323info->h245_list) {
+		GList* list2 = g_list_first(tmp_h323info->h245_list);
+		while (list2)
+		{
+			h245_address_t *h245_add=list2->data;
+			g_free((void *)h245_add->h245_address.data);
+			g_free(list2->data);
+			list2 = g_list_next(list2);
+		}
+		
+		g_list_free(tmp_h323info->h245_list);
+		
+	}
+	
+	g_free(p);
+}
 /****************************************************************************/
 /* whenever a H225 packet is seen by the tap listener */
 static int 
@@ -1749,10 +1786,12 @@ H225calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, con
 		COPY_ADDRESS(&(strinfo->initial_speaker),&(pinfo->src));
 		strinfo->selected=FALSE;
 		strinfo->first_frame_num=pinfo->fd->num;
-		strinfo->start_sec=pinfo->fd->rel_ts.secs;
+		strinfo->start_sec=(gint32) (pinfo->fd->rel_ts.secs);
 		strinfo->start_usec=pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->protocol=VOIP_H323;
 		strinfo->prot_info=g_malloc(sizeof(h323_calls_info_t));
+		strinfo->free_prot_info = free_h225_info;
+		
 		tmp_h323info = strinfo->prot_info;
 		g_assert(tmp_h323info != NULL);
 		tmp_h323info->guid = g_memdup(&pi->guid, sizeof pi->guid);
@@ -1782,7 +1821,7 @@ H225calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, con
 		COPY_ADDRESS(&(tmp_src),&(pinfo->src));
 		COPY_ADDRESS(&(tmp_dst),&(pinfo->dst));
 
-		strinfo->stop_sec=pinfo->fd->rel_ts.secs;
+		strinfo->stop_sec=(gint32) (pinfo->fd->rel_ts.secs);
 		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->last_frame_num=pinfo->fd->num;
 		++(strinfo->npackets);
@@ -1802,7 +1841,7 @@ H225calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, con
 				h245_add->h245_address.type=AT_IPv4;
 				h245_add->h245_address.len=4;
 				h245_add->h245_address.data = g_malloc(sizeof(pi->h245_address));
-				g_memmove((void *)(h245_add->h245_address.data), &(pi->h245_address), 4);
+				memcpy((void *)(h245_add->h245_address.data), &(pi->h245_address), 4);
 				h245_add->h245_port = pi->h245_port;
 				add_h245_Address(tmp_h323info, h245_add);
 			}
@@ -1980,7 +2019,7 @@ static void h245_free_labels(guint32 new_frame_num)
 }
 
 /* add the frame_label and comment to h245_labels and free the actual one if it is different frame num */ 
-static void h245_add_label(guint32 new_frame_num, gchar *frame_label, gchar *comment)
+static void h245_add_label(guint32 new_frame_num, const gchar *frame_label, const gchar *comment)
 {
 	h245_free_labels(new_frame_num);
 
@@ -2395,10 +2434,11 @@ MGCPcalls_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 		COPY_ADDRESS(&(strinfo->initial_speaker),&(pinfo->src));
 		strinfo->first_frame_num=pinfo->fd->num;
 		strinfo->selected=FALSE;
-		strinfo->start_sec=pinfo->fd->rel_ts.secs;
+		strinfo->start_sec=(gint32) (pinfo->fd->rel_ts.secs);
 		strinfo->start_usec=pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->protocol=VOIP_MGCP;
 		strinfo->prot_info=g_malloc(sizeof(mgcp_calls_info_t));
+		strinfo->free_prot_info = g_free;
 		tmp_mgcpinfo=strinfo->prot_info;
 		tmp_mgcpinfo->endpointId = g_strdup(pi->endpointId);
 		tmp_mgcpinfo->fromEndpoint = fromEndpoint;
@@ -2482,7 +2522,7 @@ MGCPcalls_packet( void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, co
 
 	comment = g_strdup_printf("MGCP %s %s%s", tmp_mgcpinfo->endpointId, (pi->mgcp_type == MGCP_REQUEST)?"Request":"Response", pi->is_duplicate?" Duplicate":"");
 
-	strinfo->stop_sec=pinfo->fd->rel_ts.secs;
+	strinfo->stop_sec=(gint32) (pinfo->fd->rel_ts.secs);
 	strinfo->stop_usec=pinfo->fd->rel_ts.nsecs/1000;
 	strinfo->last_frame_num=pinfo->fd->num;
 	++(strinfo->npackets);
@@ -2600,10 +2640,12 @@ ACTRACEcalls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_,
 			COPY_ADDRESS(&(strinfo->initial_speaker),actrace_direction?&pstn_add:&(pinfo->src));
 			strinfo->first_frame_num=pinfo->fd->num;
 			strinfo->selected=FALSE;
-			strinfo->start_sec=pinfo->fd->rel_ts.secs;
+			strinfo->start_sec=(gint32) (pinfo->fd->rel_ts.secs);
 			strinfo->start_usec=pinfo->fd->rel_ts.nsecs/1000;
 			strinfo->protocol=VOIP_AC_CAS;
 			strinfo->prot_info=g_malloc(sizeof(actrace_cas_calls_info_t));
+			strinfo->free_prot_info = g_free;
+			
 			tmp_actrace_cas_info=strinfo->prot_info;
 			tmp_actrace_cas_info->bchannel=pi->cas_bchannel;
 			tmp_actrace_cas_info->trunk=actrace_trunk;
@@ -2612,7 +2654,7 @@ ACTRACEcalls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_,
 			tapinfo->strinfo_list = g_list_append(tapinfo->strinfo_list, strinfo);
 		}
 
-		strinfo->stop_sec=pinfo->fd->rel_ts.secs;
+		strinfo->stop_sec=(gint32) (pinfo->fd->rel_ts.secs);
 		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs/1000;
 		strinfo->last_frame_num=pinfo->fd->num;
 		++(strinfo->npackets);
@@ -2677,6 +2719,359 @@ remove_tap_listener_actrace_calls(void)
 	have_actrace_tap_listener=FALSE;
 }
 
+
+
+
+
+/**************************** TAP for H248/MEGACO **********************************/
+static gboolean have_h248_tap_listener = FALSE;
+static gboolean have_megaco_tap_listener = FALSE;
+
+#define gcp_is_req(type) ( type == GCP_CMD_ADD_REQ || type == GCP_CMD_MOVE_REQ || type == GCP_CMD_MOD_REQ || \
+							type == GCP_CMD_SUB_REQ || type == GCP_CMD_AUDITCAP_REQ || type == GCP_CMD_AUDITVAL_REQ || \
+							type == GCP_CMD_NOTIFY_REQ || type == GCP_CMD_SVCCHG_REQ || type == GCP_CMD_TOPOLOGY_REQ || \
+							type == GCP_CMD_CTX_ATTR_AUDIT_REQ )
+
+static int h248_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const void *prot_info) {
+	voip_calls_tapinfo_t *tapinfo = &the_tapinfo_struct;
+	const gcp_cmd_t* cmd = prot_info;
+	GList* list;
+	voip_calls_info_t *strinfo = NULL;
+	address* mgw;
+	address* mgc;
+	gchar mgw_addr[128];
+	
+	if (cmd->ctx->id == NULL_CONTEXT || cmd->ctx->id == ALL_CONTEXTS ) {
+		return 0;
+	}
+	
+	if ( gcp_is_req(cmd->type) ) {
+		mgw = &(pinfo->dst);
+		mgc = &(pinfo->src);
+	} else {
+		mgc = &(pinfo->dst);
+		mgw = &(pinfo->src);
+	}
+	
+	address_to_str_buf(mgw, mgw_addr, 128);
+	
+	/* check whether we already have this context in the list */
+	list = g_list_first(tapinfo->strinfo_list);
+	while (list)
+	{
+		voip_calls_info_t* tmp_listinfo = list->data;
+		
+		if (tmp_listinfo->protocol == TEL_H248){
+			if (tmp_listinfo->prot_info == cmd->ctx){
+				strinfo = (voip_calls_info_t*)(list->data);
+				break;
+			}
+		}
+		list = g_list_next (list);
+	}
+	
+	if (strinfo==NULL){
+			
+		strinfo = g_malloc(sizeof(voip_calls_info_t));
+		strinfo->call_state = VOIP_NO_STATE;
+		strinfo->call_active_state = VOIP_ACTIVE;
+		strinfo->from_identity = g_strdup_printf("%s : %.8x", mgw_addr, cmd->ctx->id);
+		strinfo->to_identity = g_strdup("");
+		strinfo->prot_info = cmd->ctx;
+		strinfo->free_prot_info = NULL;
+				
+		strinfo->npackets = 1;
+		strinfo->first_frame_num=pinfo->fd->num;
+		strinfo->last_frame_num=pinfo->fd->num;
+
+		COPY_ADDRESS(&(strinfo->initial_speaker), mgc);
+	
+		strinfo->protocol = TEL_H248;
+		strinfo->call_num = tapinfo->ncalls++;
+		strinfo->start_sec=(gint32) (pinfo->fd->rel_ts.secs);
+		strinfo->start_usec=pinfo->fd->rel_ts.nsecs;
+		strinfo->stop_sec=(gint32) (pinfo->fd->rel_ts.secs);
+		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs;
+
+		strinfo->selected = FALSE;
+		
+		tapinfo->strinfo_list = g_list_append(tapinfo->strinfo_list, strinfo);
+
+	} else {
+		GString* s = g_string_new("");
+		gcp_terms_t *ctx_term;
+
+		g_free(strinfo->from_identity);
+		strinfo->from_identity = g_strdup_printf("%s : %.8x", mgw_addr, ((gcp_ctx_t*)strinfo->prot_info)->id);
+
+		g_free(strinfo->to_identity);
+
+		for (ctx_term = ((gcp_ctx_t*)strinfo->prot_info)->terms.next;
+			 ctx_term;
+			 ctx_term = ctx_term->next ) {
+			if ( ctx_term->term && ctx_term->term->str) {
+				g_string_sprintfa(s," %s",ctx_term->term->str);
+			}
+		}
+		
+		strinfo->to_identity = s->str;
+		g_string_free(s,FALSE);
+		
+		strinfo->stop_sec=(gint32) (pinfo->fd->rel_ts.secs);
+		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs;
+		strinfo->last_frame_num=pinfo->fd->num;
+		++(strinfo->npackets);
+	}
+	
+	add_to_graph(tapinfo, pinfo, cmd->str ? cmd->str : "unknown Msg",
+				 ep_strdup_printf("TrxId = %u, CtxId = %.8x",cmd->trx->id,cmd->ctx->id),
+				 strinfo->call_num, &(pinfo->src), &(pinfo->dst), 1);
+	
+	++(tapinfo->npackets);
+
+	tapinfo->redraw = TRUE;
+	
+	return 1;
+}
+
+void h248_calls_init_tap(void)
+{
+	GString *error_string;
+	
+	
+	if(have_megaco_tap_listener==FALSE)
+	{
+		error_string = register_tap_listener("megaco", &(the_tapinfo_struct.megaco_dummy),
+											 NULL,
+											 voip_calls_dlg_reset,
+											 h248_calls_packet,
+											 voip_calls_dlg_draw);
+		
+		if (error_string != NULL) {
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+						  error_string->str);
+			g_string_free(error_string, TRUE);
+			exit(1);
+		}
+		
+		have_megaco_tap_listener=TRUE;
+	}
+	
+	if(have_h248_tap_listener==FALSE)
+	{
+		error_string = register_tap_listener("h248", &(the_tapinfo_struct.h248_dummy),
+											 NULL,
+											 voip_calls_dlg_reset,
+											 h248_calls_packet,
+											 voip_calls_dlg_draw);
+		
+		if (error_string != NULL) {
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+						  error_string->str);
+			g_string_free(error_string, TRUE);
+			exit(1);
+		}
+		
+		have_h248_tap_listener=TRUE;
+	}
+}
+
+void
+remove_tap_listener_h248_calls(void)
+{
+	protect_thread_critical_region();
+	remove_tap_listener(&(the_tapinfo_struct.h248_dummy));
+	remove_tap_listener(&(the_tapinfo_struct.megaco_dummy));
+	unprotect_thread_critical_region();
+	
+	have_megaco_tap_listener=FALSE;
+	have_h248_tap_listener=FALSE;
+}
+
+
+/**************************** TAP for SCCP and SUA **********************************/
+/**************************** ( RANAP and BSSAP ) **********************************/
+static gboolean have_sccp_tap_listener = FALSE;
+static gboolean have_sua_tap_listener = FALSE;
+
+static const voip_protocol sccp_proto_map[] = {
+	TEL_SCCP,
+	TEL_BSSMAP,
+	TEL_RANAP
+};
+#define SP2VP(ap) ((ap) < SCCP_PLOAD_NUM_PLOADS ? sccp_proto_map[(ap)] : TEL_SCCP)
+const value_string* sccp_payload_values;
+
+static int sccp_calls(packet_info *pinfo, const void *prot_info) {
+	voip_calls_tapinfo_t *tapinfo = &the_tapinfo_struct;
+	const sccp_msg_info_t* msg = prot_info;
+	sccp_assoc_info_t* assoc = msg->data.co.assoc;
+	GList* list;
+	voip_calls_info_t *strinfo = NULL;
+	const gchar* label = NULL;
+	const gchar* comment = NULL;
+	/* check whether we already have this assoc in the list */
+
+	for(list = g_list_first(tapinfo->strinfo_list) ; list ; list = g_list_next (list) ) {
+		if ( ((voip_calls_info_t*)(list->data))->prot_info == assoc ){
+				strinfo = (voip_calls_info_t*)(list->data);
+				break;
+		}
+	}
+	
+	if (strinfo==NULL){
+		strinfo = g_malloc(sizeof(voip_calls_info_t));
+		strinfo->call_state = VOIP_CALL_SETUP;
+		strinfo->call_active_state = VOIP_ACTIVE;
+		if ( assoc->calling_party ) {
+			strinfo->from_identity =  g_strdup(assoc->calling_party);
+		} else {
+			strinfo->from_identity =  g_strdup("Unknown");
+		}
+		
+		if ( assoc->called_party ) {
+			strinfo->to_identity =  g_strdup(assoc->called_party);
+		} else {
+			strinfo->to_identity =  g_strdup("Unknown");
+		}
+		
+		strinfo->prot_info = (void*)assoc;
+		strinfo->free_prot_info = NULL;
+		
+		strinfo->npackets = 1;
+		strinfo->first_frame_num=pinfo->fd->num;
+		strinfo->last_frame_num=pinfo->fd->num;
+		
+		COPY_ADDRESS(&(strinfo->initial_speaker), &(pinfo->src));
+		
+		strinfo->protocol =   SP2VP(assoc->payload);
+		strinfo->start_sec=(gint32) pinfo->fd->rel_ts.secs;
+		strinfo->start_usec=pinfo->fd->rel_ts.nsecs;
+		strinfo->stop_sec=(gint32) pinfo->fd->rel_ts.secs;
+		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs;
+		
+		strinfo->selected = FALSE;
+		strinfo->call_num = tapinfo->ncalls++;
+				
+		tapinfo->strinfo_list = g_list_append(tapinfo->strinfo_list, strinfo);
+	} else {
+		
+		if ( assoc->calling_party ) {
+			if (strinfo->from_identity) g_free(strinfo->from_identity );
+			strinfo->from_identity =  g_strdup(assoc->calling_party);
+		}
+		
+		if ( assoc->called_party ) {
+			if (strinfo->to_identity) g_free(strinfo->to_identity );
+			strinfo->to_identity =  g_strdup(assoc->called_party);
+		}
+		
+		strinfo->protocol =  SP2VP(assoc->payload);
+		strinfo->stop_sec=(gint32) pinfo->fd->rel_ts.secs;
+		strinfo->stop_usec=pinfo->fd->rel_ts.nsecs;
+		strinfo->last_frame_num=pinfo->fd->num;
+		++(strinfo->npackets);
+		
+		switch (msg->type) {
+			case SCCP_MSG_TYPE_CC:
+				strinfo->call_state = VOIP_IN_CALL;
+				break;
+			case SCCP_MSG_TYPE_RLC:
+				strinfo->call_state = VOIP_COMPLETED;
+				strinfo->call_active_state = VOIP_INACTIVE;
+				break;
+			default:
+				break;
+		}
+	}
+	
+	if (msg->data.co.label) {
+		label = msg->data.co.label;
+	} else {
+		label = val_to_str(msg->type, sccp_payload_values, "Unknown(%d)");
+	}
+	
+	if (msg->data.co.comment) {
+		comment = msg->data.co.comment;
+	} else {
+		comment = "";
+	}
+	
+	add_to_graph(tapinfo, pinfo, label,(void*) comment, strinfo->call_num, &(pinfo->src), &(pinfo->dst), 1);
+	
+	++(tapinfo->npackets);
+
+	tapinfo->redraw = TRUE;
+	
+	return 1;
+}
+
+static int sccp_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const void *prot_info) {
+	sccp_payload_values = sccp_message_type_acro_values;
+	return sccp_calls(pinfo, prot_info);
+}
+
+
+static int sua_calls_packet(void *ptr _U_, packet_info *pinfo, epan_dissect_t *edt _U_, const void *prot_info) {
+	sccp_payload_values = sua_co_class_type_acro_values;
+	return sccp_calls(pinfo, prot_info);
+}
+
+
+void sccp_calls_init_tap(void)
+{
+	GString *error_string;
+	
+	if(have_sccp_tap_listener==FALSE)
+	{
+		error_string = register_tap_listener("sccp", &(the_tapinfo_struct.sccp_dummy),
+											 NULL,
+											 voip_calls_dlg_reset,
+											 sccp_calls_packet,
+											 voip_calls_dlg_draw);
+		
+		if (error_string != NULL) {
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+						  error_string->str);
+			g_string_free(error_string, TRUE);
+			exit(1);
+		}
+		
+		have_sccp_tap_listener=TRUE;
+	}
+
+	if(have_sua_tap_listener==FALSE)
+	{
+		error_string = register_tap_listener("sua", &(the_tapinfo_struct.sua_dummy),
+											 NULL,
+											 voip_calls_dlg_reset,
+											 sua_calls_packet,
+											 voip_calls_dlg_draw);
+		
+		if (error_string != NULL) {
+			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK,
+						  error_string->str);
+			g_string_free(error_string, TRUE);
+			exit(1);
+		}
+		
+		have_sua_tap_listener=TRUE;
+	}
+	
+}
+
+void
+remove_tap_listener_sccp_calls(void)
+{
+	protect_thread_critical_region();
+	remove_tap_listener(&(the_tapinfo_struct.sccp_dummy));
+	unprotect_thread_critical_region();
+	
+	have_sccp_tap_listener=FALSE;
+	have_sua_tap_listener=FALSE;
+}
+
+
 /****************************************************************************/
 /* ***************************TAP for OTHER PROTOCOL **********************************/
 /****************************************************************************/
