@@ -1,7 +1,7 @@
 /* voip_calls_dlg.c
  * VoIP calls summary addition for Wireshark
  *
- * $Id: voip_calls_dlg.c 19895 2006-11-14 20:32:29Z jmayer $
+ * $Id: voip_calls_dlg.c 21190 2007-03-25 21:56:08Z lego $
  *
  * Copyright 2004, Ericsson , Spain
  * By Francisco Alcoba <francisco.alcoba@ericsson.com>
@@ -66,6 +66,7 @@
 
 #include <epan/address.h>
 #include <epan/addr_resolv.h>
+#include <epan/dissectors/packet-h248.h>
 
 #ifdef HAVE_LIBPORTAUDIO
 #if GTK_MAJOR_VERSION >= 2
@@ -207,6 +208,8 @@ static void voip_calls_remove_tap_listener(void)
 	remove_tap_listener_h225_calls();
 	remove_tap_listener_h245dg_calls();
 	remove_tap_listener_q931_calls();
+	remove_tap_listener_h248_calls();
+	remove_tap_listener_sccp_calls();
 	remove_tap_listener_sdp_calls();
 	remove_tap_listener_rtp();
 	remove_tap_listener_rtp_event();
@@ -311,9 +314,7 @@ voip_calls_on_filter                    (GtkButton       *button _U_,
 
 	if (filter_length < max_filter_length){
 		gtk_entry_append_text(GTK_ENTRY(main_display_filter_widget), filter_string_fwd->str);
-	}
-	else{
-
+	} else {
 		g_string_free(filter_string_fwd, TRUE);
 		filter_string_fwd = g_string_new(filter_prepend);
 
@@ -358,10 +359,18 @@ voip_calls_on_filter                    (GtkButton       *button _U_,
 				g_string_sprintfa(filter_string_fwd, ") ");
 				gtk_entry_append_text(GTK_ENTRY(main_display_filter_widget), filter_string_fwd->str);
 				break;
+			case TEL_H248: {
+				const gcp_ctx_t* ctx = selected_call_fwd->prot_info;
+				gtk_entry_append_text(GTK_ENTRY(main_display_filter_widget), ep_strdup_printf("h248.ctx == 0x%x", ctx->id ));
+				break;
+			}
+			case TEL_SCCP:
 			case VOIP_MGCP:
 			case VOIP_AC_ISDN:
 			case VOIP_AC_CAS:
 			case MEDIA_T38:
+ 		        case TEL_BSSMAP:
+		        case TEL_RANAP:
 				/* XXX - not supported */
 				break;
 		}
@@ -791,7 +800,7 @@ void voip_calls_dlg_reset(void *ptr _U_)
 	voip_calls_reset((voip_calls_tapinfo_t*) voip_calls_get_info());
 
 	/* close the graph window if open */
-	if (graph_analysis_data->dlg.window != NULL) {
+	if (graph_analysis_data && graph_analysis_data->dlg.window != NULL) {
 		window_cancel_button_cb(NULL, graph_analysis_data->dlg.window);
 		graph_analysis_data->dlg.window = NULL;
 	}
@@ -823,6 +832,8 @@ voip_calls_init_tap(const char *dummy _U_, void* userdata _U_)
 	h225_calls_init_tap();
 	h245dg_calls_init_tap();
 	q931_calls_init_tap();
+	h248_calls_init_tap();
+	sccp_calls_init_tap();
 	sdp_calls_init_tap();
 	rtp_init_tap();
 	rtp_event_init_tap();
