@@ -1,6 +1,6 @@
 /* catapult_dct2000.c
  *
- * $Id: catapult_dct2000.c 20304 2007-01-04 13:25:25Z martinm $
+ * $Id: catapult_dct2000.c 21986 2007-05-29 14:38:40Z martinm $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -171,7 +171,7 @@ int catapult_dct2000_open(wtap *wth, int *err, gchar **err_info _U_)
     gint64  offset = 0;
     time_t  timestamp;
     guint32 usecs;
-    gint firstline_length;
+    gint firstline_length = 0;
     dct2000_file_externals_t *file_externals;
 
     /* Clear errno before reading from the file */
@@ -494,7 +494,8 @@ catapult_dct2000_seek_read(wtap *wth, gint64 seek_off,
     /* If get here, must have failed */
     *err = errno;
     *err_info = g_strdup_printf("catapult dct2000: seek_read failed to read/parse "
-                                "line at position %lld", seek_off);
+                                "line at position %" G_GINT64_MODIFIER "d",
+                                seek_off);
     return FALSE;
 }
 
@@ -777,13 +778,14 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     int  seconds_chars;
     char subsecond_decimals_buff[MAX_SUBSECOND_DECIMALS+1];
     int  subsecond_decimals_chars;
+    int  skip_first_byte = FALSE;
 
     gboolean atm_header_present = FALSE;
 
     /* Read context name until find '.' */
     for (n=0; linebuff[n] != '.' && (n < MAX_CONTEXT_NAME) && (n+1 < line_length); n++)
     {
-        if (!isalnum(linebuff[n]) && (linebuff[n] != '_'))
+	if (!isalnum((int)linebuff[n]) && (linebuff[n] != '_'))
         {
             return FALSE;
         }
@@ -809,7 +811,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
          (linebuff[n] != '/') && (port_digits <= MAX_PORT_DIGITS) && (n+1 < line_length);
          n++, port_digits++)
     {
-        if (!isdigit(linebuff[n]))
+	if (!isdigit((int)linebuff[n]))
         {
             return FALSE;
         }
@@ -836,7 +838,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
          (linebuff[n] != '/') && (protocol_chars < MAX_PROTOCOL_NAME) && (n < line_length);
          n++, protocol_chars++)
     {
-        if (!isalnum(linebuff[n]) && linebuff[n] != '_')
+	if (!isalnum((int)linebuff[n]) && linebuff[n] != '_')
         {
             return FALSE;
         }
@@ -860,10 +862,10 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
 
     /* Following the / is the variant number.  No digits indicate 1 */
     for (variant_digits = 0;
-         (isdigit(linebuff[n])) && (variant_digits <= MAX_VARIANT_DIGITS) && (n+1 < line_length);
+         (isdigit((int)linebuff[n])) && (variant_digits <= MAX_VARIANT_DIGITS) && (n+1 < line_length);
          n++, variant_digits++)
     {
-        if (!isdigit(linebuff[n]))
+	if (!isdigit((int)linebuff[n]))
         {
             return FALSE;
         }
@@ -892,11 +894,11 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
         n++;
 
         for (outhdr_chars = 0;
-             (isdigit(linebuff[n]) || linebuff[n] == ',') &&
+             (isdigit((int)linebuff[n]) || linebuff[n] == ',') &&
              (outhdr_chars <= MAX_OUTHDR_NAME) && (n+1 < line_length);
              n++, outhdr_chars++)
         {
-            if (!isdigit(linebuff[n]) && (linebuff[n] != ','))
+	    if (!isdigit((int)linebuff[n]) && (linebuff[n] != ','))
             {
                 return FALSE;
             }
@@ -916,7 +918,9 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     /******************************************************************/
     /* Now check whether we know how to use a packet of this protocol */
 
-    if ((strcmp(protocol_name, "ip") == 0)  || (strcmp(protocol_name, "sctp") == 0))
+    if ((strcmp(protocol_name, "ip") == 0) ||
+        (strcmp(protocol_name, "sctp") == 0) ||
+        (strcmp(protocol_name, "mipv6") == 0))
     {
         *encap = WTAP_ENCAP_RAW_IP;
     }
@@ -940,6 +944,13 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
             atm_header_present = TRUE;
         }
     }
+    else if (strcmp(protocol_name, "fpiur_r5") == 0)
+    {
+        /* FP (IuR) over AAL2 */
+        *encap = WTAP_ENCAP_ATM_PDUS_UNTRUNCATED;
+        atm_header_present = TRUE;
+    }
+
 
     else
     if (strcmp(protocol_name, "ppp") == 0)
@@ -949,7 +960,13 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     else
     if (strcmp(protocol_name, "isdn_l3") == 0)
     {
-        /* Despite the name, this does seem to correspond to L2... */
+       /* TODO: find out what this byte means... */
+        skip_first_byte = TRUE;
+        *encap = WTAP_ENCAP_ISDN;
+    }
+    else
+    if (strcmp(protocol_name, "isdn_l2") == 0)
+    {
         *encap = WTAP_ENCAP_ISDN;
     }
     else
@@ -1005,7 +1022,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
 
         /* Read consecutive hex chars into atm header buffer */
         for (;
-             (isalnum(linebuff[n]) &&
+             (isalnum((int)linebuff[n]) &&
               (n < line_length) &&
               (header_chars_seen < AAL_HEADER_CHARS));
              n++, header_chars_seen++)
@@ -1056,7 +1073,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     /* Find and read the timestamp                                       */
 
     /* Now scan to the next digit, which should be the start of the timestamp */
-    for (; !isdigit(linebuff[n]) && (n < line_length); n++);
+    for (; !isdigit((int)linebuff[n]) && (n < line_length); n++);
     if (n >= line_length)
     {
         return FALSE;
@@ -1071,7 +1088,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
          (n < line_length);
          n++, seconds_chars++)
     {
-        if (!isdigit(linebuff[n]))
+        if (!isdigit((int)linebuff[n]))
         {
             /* Found a non-digit before decimal point. Fail */
             return FALSE;
@@ -1103,7 +1120,7 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
          (n < line_length);
          n++, subsecond_decimals_chars++)
     {
-        if (!isdigit(linebuff[n]))
+        if (!isdigit((int)linebuff[n]))
         {
             return FALSE;
         }
@@ -1141,15 +1158,14 @@ gboolean parse_line(gint line_length, gint *seconds, gint *useconds,
     /* Set number of chars that comprise the hex string protocol data */
     *data_chars = line_length - n;
 
-    /* Need to skip first byte (2 hex string chars) from ISDN messages.
-       TODO: find out what this byte means...
-    */
-    if (*encap == WTAP_ENCAP_ISDN)
+    /* May need to skip first byte (2 hex string chars) */
+    if (skip_first_byte)
     {
         *data_offset += 2;
         *data_chars -= 2;
     }
 
+
     return TRUE;
 }
 
@@ -1270,7 +1286,7 @@ void set_aal_info(union wtap_pseudo_header *pseudo_header, packet_direction_t di
 
     /* cid is usually last byte.  Unless last char is not hex digit, in which
        case cid is derived from last char in ascii */
-    if (isalnum(aal_header_chars[11]))
+    if (isalnum((int)aal_header_chars[11]))
     {
         pseudo_header->dct2000.inner_pseudo_header.atm.aal2_cid =
             ((hex_from_char(aal_header_chars[10]) << 4) |
@@ -1364,7 +1380,7 @@ gint wth_equal(gconstpointer v, gconstpointer v2)
 /***********************************************/
 guint wth_hash_func(gconstpointer v)
 {
-    return (guint)v;
+    return (guint)(unsigned long)v;
 }
 
 
